\ Umbilical Forth target control file for GNC Cardmaster Z80

((
Released for use with the MPE Forth Cross Compiler by:

MicroProcessor Engineering
133 Hill Lane
Shirley
Southampton SO1 5AF
England

tel: (+44) 703 631441
fax: (+44) 703 339691

May 1991	Original release of v2
Feb 1994	Version 3 release for use with 5.1 compilers up 

))
\ Z80 ROM Umbilical Forth target

only forth definitions decimal

\
\ Select which NEXT macro to use, the one that generates a short NEXT
\ which assembles (IX) JP, or a fast NEXT that compiles NEXT as in-line code.
\
  all from-file nextjp			\ macro for short NEXT
\ all from-file nextfast		\ macro for fast NEXT

\ Insert other macros here

\
\ Using Umbilical Forth with an EPROM emulator is a good way to work,
\ but finally an EPROM image file must be generated.
\ Select only one output device.

  hex 0320 emu-base			\ uncomment for output to emulator
  e27256 8bit output-emulator

\ The cross compiler can now be switched on

CROSS-COMPILE

\  no-log				\ no log
  no-heads				\ no dictionary

only forth definitions			\ initial target search order

-1 equ umbilical?			\ true for umbilcal forth
					\ zero for standalone

\ define memory layout

hex
      $0000 $6000   KERNEL GNCCM
      $8000 $8fff 0 KERNEL-RAM GNCCM-DATA
      $FFFF MEM-END

      $0000 IN-EMULATOR GNCCM

      0C000 equ appl-rom
      0FFFF equ appl-rom-end

use-code GNCCM
use-data GNCCM-DATA

\ define memory allocations
\ these may need to be modified to run on a single chip system

$0001 equ #tasks			\ cannot use M/T in this model

$0080 equ User-size			\ size of tasks user area
$0040 equ SP-size 			\ size of tasks data stack
$0040 equ RP-size			\ size of tasks return stack
$0100 equ intram			\ space used by interrupt page

$0080 EQU Tib-len			\ terminal i/p buffer length

\ calculate memory map

User-size SP-size + equ Task-s0		\ initial offset of data stack					  
Task-s0   RP-size + equ Task-r0		\ initial offset of return stack					  

Task-r0 equ per-task			\ system area size for each task

Sp-size 2/ equ max-depth		\ max number of cells in data stack

EM      \ length            name
  intram - 		dup equ INT-INIT-U0 	\ base of interrupt area
  #tasks per-task * -	dup equ INIT-U0		\ base of task user area
  tib-len - 		dup equ INIT-TIB	\ base of TIB
drop

\ Place the stacks inside the first task area
init-u0 Task-r0 + equ INIT-R0
init-u0 Task-s0 + equ INIT-S0

\ RAM allocation
\ If Umbilical Forth is being used on a true single chip system
\ RAM allocation must be very tightly controlled.
\
there equ UP         2 allot-ram        \ user area pointer
there equ rx-avail   1 allot-ram        \ character available
there equ rx-char    1 allot-ram        \ character read
there equ rx-error   1 allot-ram        \ last error

\ pull in the files

  decimal all from-file messages	\ message equates for Umbilical Forth

  decimal all from-file startZ80	\ Z80 startup code
  decimal all from-file minZ80		\ code words
  decimal all from-file drivers\dartctci \ interrupt driver code for DART/CTC
  decimal all from-file kernel		\ High level words

umbilical?
if(
  decimal all from-file targend    	\ message driver
)endif

\ cold - simple example application

hex

: cold
  init-ser                              \ initialise UART
  0A base c!
  cr cr ." MicroProcessor Engineering Umbilical Forth"
  begin  cr
         0A 0
         do  i 030 + emit  loop
         cr ." Press any key to continue " key drop
  again  ;                              \ something silly


\ Define what the system will do at power up.
\ For the final application, TARGEND.FTH need not be included 
\ when the interactive Umbilical Forth is no longer needed.
\ RUN-UMBILICAL is defined in TARGEND.FTH.

umbilical?
if(
  make-turnkey run-umbilical		\ cold start to Umbilical Forth
)else(
  make-turnkey cold			\ cold start to application
)endif

umbilical?
if(
  UMBILICAL-FORTH
)else(
  FINIS
)endif
