\ I/O words for single chip 64180 ASCI systems

((
Copyright (C) 1990

MicroProcessor Engineering
133/133a Hill Lane
Shirley
Southampton SO1 5AF

tel: (+44) 703 631441
fax: (+44) 703 339691

Note that CTS0 and DCD0 must be held low for the 64180 ASCI0 UART to work!

The serial driver for an Umbilical Forth system must put the processor
to sleep while waiting for a character from the serial line. This is required
to allow an EPROM emulator to write data into the EPROM space. By using
an interrupt driven serial line, and putting the processor to sleep while
waiting, no EPROM memory accesses take place.

If the processor cannot be put to sleep, or a bit-banging UART is being
used, the polling loop must be executed from RAM to avoid accessing
EPROM space.

May 1991	Converted to text files
		Drivers moved to DRIVERS directory
))
\ ASCI vectors                                     sfp 30/03/90

hex

\ Define address used as base of ASCI interrupt vectors.
\ first define the base of the 64180 internal vectors, IL reg
int-page 020 + equ internal-vectors

\ the ASCI0 vectors through a location 0E bytes above this
internal-vectors 0E + equ ASCI0-vector

\ ASCI - init-ASCI0
\
assembler
l: init-ASCI0   \ -- ; initialise 64180 ASCI0
  a, # 0 ld   RCR a out0   DCNTL a out0 \ no refresh, no waits
  rx-avail a ld   rx-error a ld         \ reset flags
  a, # 064 ld   CNTLA0 a out0           \ Enable Trans. & Rec.
  a, # 2 ld   CNTLB0 a out0             \ SET BAUD RATE, Div:=16 SS:=2=/4
  a, # 08 ld   STAT0 a out0             \ enable receiver int.
  ret                                   \
forth                                   \
\ baud rate numbers  0=38400, 1=19200, 2=9600, 3=4800, 4=2400 .....

code init-ser   \ --
  di
  init-ASCI0 call                       \ initialise 64180 ASCI0
  2 im                                  \ mode 2 interupts
  a, # int-page .hi ld   i, a ld        \ set I and IL
  a, # internal-vectors .lo ld   IL a out0
  ei                                    \ enable ints
  next,   end-code                      \ all done

\ (emit)                                           sfp 31/03/90

hex

code (emit)     \ char -- ; transmit character primitive
  begin,
    a, STAT0 in0   a, # 2 and           \ extract TX status bit
  nz, until,                            \ spin until set
  hl pop   a, l ld   TDR0 a out0        \ transmit character
  next,
end-code

\
\ Receiver interrupt
\
assembler
l: rx-int
  af push
  a, STAT0 in0                          \ get status
  a, # 0F0 and   rx-error a ld          \ save error code
  a, TSR0 in0   rx-char a ld            \ read character
  a, # -1 ld   rx-avail a ld            \ set flag
  af pop   ei   reti
forth
rx-int ASCI0-vector !

\
\ ASCI - (key) (key?)
\
code (key)      \ -- char ; wait for character
  begin,
    a, rx-avail ld   a, a or
    z, while,  slp			\ put processor to sleep while waiting
  repeat,
  a, rx-char ld   l, a ld   h, # 0 ld
  a, # 0 ld   rx-avail a ld
  hl push   next,   end-code

code (key?)     \ -- t/f ; return true if character ready
  a, rx-avail ld   h, a ld   l, a ld
  hl push   next,   end-code
\ Umbilical Forth i/o

hex

umbilical? if(
\
\ These definitions are for use when debugging with Umbilical Forth.
\ The definitions of KEY, KEY?, and EMIT send output to the PC.
\
: key?          \ -- t/f
  key?-cmnd>h send-message  ;

: key           \ -- char
  key-cmnd>h send-message  ;

: emit          \ char -- ; send character
  emit-cmnd>h send-message  out c@ 1+  out c!  ;
)else(
\
\ These definitions of KEY, KEY, and EMIT are for standalone use without
\ the Umbilical Forth monitor.
\
: emit		\ char --
  (emit)  out c@ 1  out c! ;

: key?		\ -- t/f
  (key?)  ;

: key		\ -- char
  (key)  ;
)endif

: type          \ addr len -- ; display string
  bounds ?do  i c@ emit  loop  ;

: cr            \ --
  0A emit   0D emit  0 out c!  ;

\ all done

decimal

;p
