\ I/O words for Z80+DART+CTC for GNC CardMaster

((
Copyright (C) 1990

MicroProcessor Engineering
133/133a Hill Lane
Shirley
Southampton SO1 5AF

tel: (+44) 703 631441
fax: (+44) 703 339691

The serial driver for an Umbilical Forth system must put the processor
to sleep while waiting for a character from the serial line. This is required
to allow an EPROM emulator to write data into the EPROM space. By using
an interrupt driven serial line, and putting the processor to sleep while
waiting, no EPROM memory accesses take place.

If the processor cannot be put to sleep, or a bit-banging UART is being
used, the polling loop must be executed from RAM to avoid accessing
EPROM space.

May 1991	Converted to text files
		Drivers moved to DRIVERS directory
))
\ DART vectors                                     sfp 30/03/90

hex

\
\ Define address used as base of DART interrupt vectors.
\ 16 bytes are needed for the 8 vectors.
\
int-page 020 + equ dart-vector

\
\ Define addresses for DART and CTC
\
020		equ dart		\ Z80 DART chip base address
dart            equ a-data              \ CH A is RS422
dart 1+         equ b-data              \ CH B is RS232
dart 2+         equ a-cmd
dart 3 +        equ b-cmd
01              equ rda                 \ data available bit
04              equ tbe                 \ transmit buffer bit

030             equ ctc                 \ Z80 CTC chip base address
ctc             equ ctc0                \ CH 0 for RS422
ctc 1+          equ ctc1                \ CH 1 for RS232
ctc 2+          equ ctc2                \ CH 2 for single step
ctc 3 +         equ ctc3                \ CH 3 for MCP

\ initialisation routines

hex

\
\ INIT-CTC initialises the CTC
\
assembler
l: init-ctc
  a, # 057 ld   ctc1 a out              \ initialse CTC1
  a, # 0D ld   ctc1 a out               \ set CTC1 to 9600 baud
  ret
forth

\
\ INIT-DART sets up the DART, including the interrupt vector
\
assembler
l: init-dart
  a, # 018 ld   b-cmd a out             \ reset DART channel B
  a, # 2 ld   b-cmd a out               \ select WR2
  a, # dart-vector .lo ld   b-cmd a out \ set interrupt vector
  a, # 4 ld   b-cmd a out   a, # 044 ld   b-cmd a out
  a, # 3 ld   b-cmd a out   a, # 0C1 ld   b-cmd a out
  a, # 5 ld   b-cmd a out   a, # 068 ld   b-cmd a out
  a, # 1 ld   b-cmd a out   a, # 014 ld   b-cmd a out
  a, # 0 ld   rx-avail a ld   rx-error a ld
  ret
forth

\
\ INIT-SER is the Forth word called to initialise the serial port.
\
code init-ser                           \ -- ; initialise serial
  di                                    \ no interrupts
  init-ctc call                         \ deal with CTC
  init-dart call                        \ deal with DART
  2 im                                  \ interrupt mode 2
  a, # dart-vector .hi ld   i, a ld     \ set vector page
  ei                                    \ enable mode 2 ints
  next,   end-code                      \ all done
\ DART - (emit) 

hex

code (emit)	\ char -- ; transmit character primitive
l: char-op
  a, # 0 ld   b-cmd a out   nop         \ select reg
  a, b-cmd in   a, # tbe and   z, char-op jr \ spin until ready
  hl pop   a, l ld   b-data a out       \ transmit character
  next,   end-code

\
\ Receiver interrupt
\
assembler
l: rx-int       \ main receiver interrupt
  af push
  a, b-data in   rx-char a ld   \ read and save character
  a, # -1 ld   rx-avail a ld    \ set flag
  af pop   ei   reti
forth

\
\ Error interrupt
\
assembler
l: src-int      \ entered after an error
  af push
  a, # 01 ld   b-cmd a out   nop        \ select reg 1
  a, b-cmd in   rx-error a ld           \ read it for error type
  a, # 030 ld   b-cmd a out             \ issue error reset
  a, b-data in   rx-char a ld           \ read & save character
  a, # 0FF ld   rx-avail a ld           \ set flag
  af pop   ei   reti
forth

rx-int dart-vector 4 + !        \ receiver interrupt
src-int dart-vector 6 + !

\ DART - (key) key .....................           sfp 29/03/90

hex

code (key)      \ -- char ; read character primitive
  begin,  a, rx-avail ld   a, a or      \ test flag
    z, while,  halt                     \ stop while no flag
  repeat,
  h, # 0 ld   a, rx-char ld   l, a ld   \ form character
  a, # 0 ld   rx-avail a ld             \ reset flag
  hl push   next,   end-code            \ return character

code (key?)     \ -- t/f
  a, rx-avail ld   h, a ld   l, a ld    \ get flag
  hl push   next,   end-code            \ return it

\ Umbilical Forth i/o
hex

umbilical? if(
\
\ These definitions are for use when debugging with Umbilical Forth.
\ The definitions of KEY, KEY?, and EMIT send output to the PC.
\
: key?          \ -- t/f
  key?-cmnd>h send-message  ;

: key           \ -- char
  key-cmnd>h send-message  ;

: emit          \ char -- ; send character
  emit-cmnd>h send-message  out c@ 1+  out c!  ;
)else(
\
\ These definitions of KEY, KEY, and EMIT are for standalone use without
\ the Umbilical Forth monitor.
\
: emit		\ char --
  (emit)  out c@ 1  out c! ;

: key?		\ -- t/f
  (key?)  ;

: key		\ -- char
  (key)  ;
)endif

: type          \ addr len -- ; display string
  bounds ?do  i c@ emit  loop  ;

: cr            \ --
  0A emit   0D emit  0 out c!  ;
\ all done

decimal

;p


