\ MPE 64180 Single Chip Forth Code definitions       sfp 21/03/90

((
Copyright (c) 1990-1991
MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF

tel: (+44) 703 631441
fax: (+44) 703 339691

March 1990	Original issue:

May 1991	converted to text file
		converted to v2.0 Umbilical Forth
))
\ dodoes docreate next execute lit ?branch branch execute (of)

assembler
l: dodoes       iy dec   0 (iy), b ld   iy dec   0 (iy), c ld
                bc pop
l: docreate     hl inc   hl inc   hl inc   hl push
l: next         a, (bc) ld    bc inc    l, a ld
                a, (bc) ld    bc inc    h, a ld
                (hl) jp
forth

\ execute lit branch ?branch [of]                  sfp 18/04/87

code execute	\ cfa -- ; execute word whose CFA is given
  hl pop   (hl) jp   end-code


internal	\ The following code is compiled without headers

code lit	\ -- n ; returns an in-line literal
  a, (bc) ld   bc inc   l, a ld		\ BC is Forth IP
  a, (bc) ld   bc inc   h, a ld
  hl push   next,
end-code

code branch	\ -- ; branch to address given in-line
l: bran1
  h, b ld   l, c ld			\ BC is Forth IP
  c, (hl) ld   hl inc   b, (hl) ld	\ reload BC with target address
  next,
end-code

code ?branch	\ flag -- ; takes branch if flag=0
  hl pop				\ get flag
  a, l ld   a, h or   z, bran1 jr	\ test flag, take branch if 0
l: 0bran1
  bc inc   bc inc   next,		\ don't branch, skip over in-line data
end-code

code (of)	\ n1 n2 -- (n1=n2); n1 n2 -- n1 (n1<>n2)
  hl pop  de pop  a, a or  hl, de sbc	\ test n1=n2>
  z, 0bran1 jr   de push   bran1 jr	\ yes - skip; no - put n1 back, branch
end-code

external
\ (loop) (+loop) (do) (?do)

internal

CODE (LOOP)	\ -- ; compiled by LOOP
  0 (iy) inc   nz, bran1 jr		\ increment limit-index+8000
  1 (iy) inc   nz, bran1 jr		\  exit when l-i+8000 passes 0
L: LOOP-EXIT
  de, # 6 ld   iy, de add   0bran1 jr	\ drop 3 items from r. stack
end-code

CODE (+LOOP)	\ n -- ; compiled +LOOP
  e, 0 (iy) ld   d, 1 (iy) ld		\ fetch limit-index+8000
  hl pop   a, h ld   a, a or		\ inspect sign of increment
  m, xploo1 jp				
  hl, de add				\ +ve: add in increment
  ncy, loop-branch jr   loop-exit jr
L: XPLOO1
  hl, de add   ncy, loop-exit jr	\ -ve: add in increment
L: LOOP-BRANCH
  0 (iy), l ld  1 (iy), h ld   bran1 jr	\ restore limit-index+8000
end-code

CODE (DO)	\ limit start -- ; compiled by DO
l: pdo1
  DE, # -6 LD   IY, DE ADD		\ 3 more items on return stack
  a, (bc) ld   4 (iy), a ld   bc inc	\ fetch loop end address
  a, (bc) ld   5 (iy), a ld   bc inc	\ save on return stack
  hl pop   de pop			\ hl=start, de=limit
  2 (iy), e ld   3 (iy), d ld		\ limit on return stack
  a, a or   hl, de sbc			\ start-limit
  0 (iy), l ld   1 (iy), h ld		\ on return stack
  next,
end-code

CODE (?DO)	\ limit start -- ; compiled by ?DO
  hl pop   de pop   de push   hl push	\ hl = start, de=limit
  a, a or   hl, de sbc   nz, pdo1 jr	\ do loop if different
  hl pop   de pop   bran1 jp		\ otherwise skip it
end-code

external
\ i min max

code i		\ -- n ; return index of innermost DO ... LOOP
  e, 0 (iy) ld   d, 1 (iy) ld           \ start-limit+inc
  l, 2 (iy) ld   h, 3 (iy) ld           \ limit
  hl, de add   hl push                  \ start+inc
  next,
end-code

code min	\ n1 n2 -- n1|n2
  exx   de pop   bc pop			\ de=n2, bc=n1
  h, d ld   l, e ld			\ trial subtraction
  a, a or   hl, bc sbc
  m, min1 jp   po, min3 jp   min2 jr
l: min1
  pe, min3 jp
l: min2
  de push   exx   next,
l: min3
  bc push   exx   next,
end-code

code max
  exx   de pop   bc pop
  h, d ld   l, e ld
  a, a or   hl, bc sbc
  m, max1 jp   po, max3 jp   max2 jr
l: max1
  pe, max3 jp
l: max2
  bc push   exx   next,
l: max3
  de push   exx   next,
end-code
\ um* * udiv32/16 um/mod

code um*	\ u1 u2 -- ud ; unsigned mixed magnitude multiply
  exx   bc pop   de pop                 \ will use HL as acc.
  h, c ld   l, e ld   hl mlt   hl push  \ low word of result
  h, b ld   l, d ld   hl mlt   hl push  \ high word of result
  ix, # 0 ld   ix, sp add               \ get stack pointer
  h, b ld   l, e ld   hl mlt            \ intermediatge word 1/2
  a, l ld  a, 3 (ix) add  3 (ix), a ld  \ add to msb of low word
  a, h ld  a, 0 (ix) adc  0 (ix), a ld  \ add to lsb of hi word
  a, # 0 ld  a, 1 (ix) adc  1 (ix), a ld  \ propagate carry
  h, c ld   l, d ld   hl mlt            \ intermediate word 2/2
  a, l ld  a, 3 (ix) add  3 (ix), a ld  \ add to msb of low word
  a, h ld  a, 0 (ix) adc  0 (ix), a ld  \ add to lsb of hi word
  a, # 0 ld  a, 1 (ix) adc  1 (ix), a ld  \ propagate carry
  ix, # next ld				\ reset IX as NEXT pointer
  exx   next,				\ all done
end-code

code *		\ n1 n2 -- n3 ; signed mixed magnitude multiply
  exx   bc pop   de pop                 \ get parameters
  h, c ld   l, e ld   hl mlt   hl push  \ n2l*n1l & save it
  a, h ld                               \ preserve high part
  h, c ld   l, d ld   hl mlt   a, l add \ n2.l*n1.h & add in
  h, b ld   l, e ld   hl mlt   a, l adc \ n2.h*n1.l & add in
  hl pop   h, a ld   hl push            \ replace high part
  exx   next,				\ all done
end-code

: M*            2DUP XOR >R ABS SWAP ABS UM* R> ?dnegate  ;

hex

assembler
l: udiv32/16	\ hl:de / bc ; 32/16 unsigned divide - main divide routine
  A, # 010 LD                           \ set up loop counter
  begin,  e sla   d rl   l rl   h rl    \ shift HL:DE
          af push                       \ save sarry
          a, a or   hl, bc sbc          \ subtract divisor
          ncy, if,  e inc   af pop      \ increment quotient
               else,  af pop            \ carry = old bit 31
                      cy, if,  e inc         \ set, so still ok
                          else,  hl, bc add  \ clear, undo
                          endif,
               endif,
           a dec                        \ count down
  z, until,                             \ until done
  ret					\ de=quot, hl=rem
forth

code um/mod	\ u31 u16 -- urem uquot ; 32/16 unsigned divide
  exx   bc pop   hl pop   de pop        \ get parameters
  a, l ld   a, c sub                    \ test for overflow
  a, h ld   a, b sbc                    \ safe if bc > hl
  ncy, if,  hl, # 0ffff ld		\ no, return oflow
	    de, # 0ffff ld
       else,  udiv32/16 call            \ call main divide subr.
       endif,
  hl push   de push                     \ return results
  exx   next,
end-code
\ m/mod /mod / mod

code m/mod                              \ d n -- rem quot
  exx   bc pop   hl pop   de pop        \ get parameters
l: mod-com                              \ common code point
  a, b ld   a, h xor   af push          \ save sign of result
  bc push   a, b ld   a, a or           \ save divisor, test sgn
  m, if,  a, # 0 ld  a, c sub  c, a ld  \ make divisor +ve
          a, # 0 ld  a, b sbc  b, a ld
     endif,
  a, h ld   a, a or                     \ test sign of hl:de
  m, if,  a, # 0 ld  a, e sub  e, a ld  \ make positive
          a, # 0 ld  a, d sbc  d, a ld
          a, # 0 ld  a, l sbc  l, a ld
          a, # 0 ld  a, h sbc  h, a ld
     endif, 

  a, l ld   a, c sub                    \ test for overflow
  a, h ld   a, b sbc   ncy, moflow jp   \ safe if bc > hl
  udiv32/16 call  bc pop  a, b ld  a, a or  \ test divisor
  m, if,  a, # 0 ld  a, l sub  l, a ld  \ make remainder -ve
          a, # 0 ld  a, h sbc  h, a ld  \
     endif,                             \
  af pop                                \ test sign of quotient
  m, if,  a, # 0 ld  a, e sub  e, a ld  \ make negative
          a, # 0 ld  a, d sbc  d, a ld  \
          a, h ld   a, l or             \ remainder non-zero?
          nz, if,  de dec               \ make quotient more -ve
                   bc push   hl push   bc pop   hl pop
                   a, a or   hl, bc sbc \ rem:=divisor-rem
              endif,
     endif,

l: mexit
  hl push   de push                     \ return results
  exx   next,                           \ switch back

l: moflow
  bc pop   b rl                         \ return remainder
  cy, if,  hl, # -07fff ld  else,  hl, # 07fff ld  endif,
  af pop                                \ return quotient
  m, if,  de, # -07fff ld  else,  de, # 07fff ld  endif,
  mexit jr
end-code

code /mod                               \ n1 n2 -- rem quot
  exx   bc pop   hl, # 0 ld   de pop    \ get parameters
  a, d ld   a, a or                     \ see if negative
  m, if,  hl dec  endif,                \ if so extend sign
  mod-com jp   end-code                 \ join common code

: /             /mod nip  ;

: mod           /mod drop  ;

: mu/mod	\ ud n -- urem udquot ; used in number conversion
  >r 0 r@ um/mod r> swap >r um/mod r> ;

\ and or xor not

code and	\ n1 n2 -- n3 ; bitwise AND
  de pop   hl pop			\ get parameters
  a, e ld   a, l and   l, a ld		\ operate
  a, d ld   a, h and   h, a ld
  hl push   next,			\ return result
end-code

code or		\ n1 n2 -- n3 ; bitwise OR
  de pop   hl pop			\ get parameters
  a, e ld   a, l or   l, a ld		\ operate
  a, d ld   a, h or   h, a ld
  hl push   next,			\ return result
end-code

code xor	\ n1 n2 -- n3 ; bitwise XOR
  de pop   hl pop			\ get parameters
  a, e ld   a, l xor   l, a ld		\ operate
  a, d ld   a, h xor   h, a ld
  hl push   next,			\ return result
end-code

code not	\ n1 -- n2 ; bitwise NOT
  hl pop
  a, l ld   cpl   l, a ld
  a, h ld   cpl   h, a ld
  hl push   next,
end-code

\ sp@ sp! rp@ rp! exit >r r> r@

code sp@	\ -- n ; return data stack pointer
  hl, # 0 ld   hl, sp add   hl push   next,
end-code

code sp!	\ n -- ; set data stack pointer
  hl pop   sp, hl ld   next,
end-code

code rp@	\ -- n ; get return stack pointer
  iy push   next,
end-code

code rp!	\ n -- ; set return stack pointer
  iy pop   next,
end-code

code exit	\ -- ; addr r-- ; leave colon definition, compiled by ;
  c, 0 (iy) ld   iy inc			\ reload BC (IP) from return stack
  b, 0 (iy) ld   iy inc
  next,
end-code

code >r		\ n -- ; r-- n 
  de pop   iy dec   iy dec		\ get n, make room on return stack
  0 (iy), e ld   1 (iy), d ld   next,	\ put n on return stack
end-code

code r>		\ -- n ; n r--
  e, 0 (iy) ld   iy inc			
  d, 0 (iy) ld   iy inc
  de push   next,
end-code

code r@		\ -- n ; n r-- n
  l, 0 (iy) ld   h, 1 (iy) ld
  hl push   next,
end-code
\ 0= 0<> = <> 0< 0> u< u>

code 0=		\ n -- flag ; true if n=0
  hl pop   a, l ld   a, h or   nz, set-fls jr
l: set-tru
  hl, # -1 ld   hl push   next,
l: set-fls
  hl, # 0 ld   hl push   next,
end-code

code 0<>	\ n -- flag ; true if n<>0
  hl pop   a, l ld   a, h or
  nz, set-tru jr   set-fls jr
end-code

code =		\ n1 n2 -- flag ; true if n1=n2
  hl pop   de pop   a, a or   hl, de sbc
  z, set-tru jr   set-fls jr
end-code

code <>		\ n1 n2 -- flag ; true if n1 <>n2
  hl pop   de pop   a, a or   hl, de sbc
  nz, set-tru jr  set-fls jr
end-code

code 0<		\ n -- flag ; true if n<0
  hl pop   h rl
l: set-t/f
  a, a sbc   h, a ld   l, a ld
  hl push   next,
end-code

code 0>		\ n -- flag ; true if n>0
  hl pop 				\ get n
  a, h ld   a, a or   m, set-fls jp	\ is n negative, FALSE
  a, l or   z, set-fls jr		\ is n zero, FALSE
  set-tru jr				\ otherwise TRUE
end-code

code u<		\ n1 n2 -- flag ; true if unsigned n1<n2
  de pop   hl pop   a, a or   hl, de sbc   set-t/f jr
end-code

code u>		\ n1 n2 -- flag ; true if unsigned n1>n2
  hl pop   de pop   a, a or   hl, de sbc   set-t/f jr
end-code
\ < > d< d> d0= d=

code <		\ n1 n2 -- flag ; true if signed n1<n2
   de pop   hl pop   a, a or		\ get parameters, clear carry
l: set-m/p
   hl, de sbc   m, <1 jp   po, <3 jp	\ comparison must test sign and o/flow
   <2 jr
l: <1
   pe, <3 jp
l: <2
   hl, # -1 ld   hl push   next,
l: <3
   hl, # 0 ld   hl push   next,
end-code

code >		\ n1 n2 -- flag ; true if signed n1>n2
  hl pop   de pop   a, a or   set-m/p jr
end-code

: <=		\ n1 n2 -- t/f ; true if n1 <= n2
  > not
;

: >=		\ n1 n2 -- t/f ; true if n1 >= n2
  < not
;

\ d< d>                                            sfp 16/04/87
((
code d<		\ d1 d2 -- t/f
  de pop   exx   de pop   exx
  hl pop   exx   hl pop
  a, a or   hl, de sbc   exx   set-m/p jr
end-code

code d>         \ d1 d2 -- t/f
  hl pop   exx   hl pop   exx
  de pop   exx   de pop
  a, a or   hl, de sbc   exx   set-m/p jr
end-code
))
((
code d=		\ d1 d2 -- t/f
  hl pop   exx   hl pop   exx
  de pop   exx   de pop
  a, a or   hl, de sbc   exx   nz, set-fls jp
  a, a or   hl, de sbc   nz, set-fls jp   set-tru jp
end-code

code d0=	\ d -- t/f
  hl pop   de pop
  a, h ld   a, l or   a, d or   a, e or
  z, set-tru jp   set-fls jp
end-code
))
\ negate ?negate abs                               sfp 18/04/87

code negate	\ n -- -n
l: neg1
  de pop
  hl, # 0 ld   a, a or   hl, de sbc
  hl push   next,
end-code

code ?negate	\ n1 n2 -- n1' ; negate n1 if n2 negative
  hl pop   h sla   cy, neg1 jr   next,
end-code

code abs	\ n -- |n|
  hl pop   hl push   h sla   cy, neg1 jr   next,
end-code


\ dnegate ?dnegate dabs                            sfp 18/04/87

code dnegate	\ d1 -- -d1
l: dneg1
  exx   bc pop   de pop                 \ switch regs, get param
  a, a or                               \ reset carry flag
  hl, # 0 ld   hl, de sbc   hl push     \ negate low word
  hl, # 0 ld   hl, bc sbc   hl push     \ negate high word
  exx   next,				\ all done
end-code

code ?dnegate	\ d n -- d' ; negate d if n negative
  hl pop   h sla   cy, dneg1 jr   next,
end-code

code dabs	\ d -- |d|
  hl pop   hl push   h sla   cy, dneg1 jr   next,
end-code

\ over 2over

code over	\ n1 n2 -- n1 n2 n1
  de pop   hl pop   hl push
  de push   hl push   next,
end-code

code 2over	\ n1 n2 n3 n4 -- n1 n2 n3 n4 n1 n2
  exx   af pop   de pop   bc pop   hl pop
  hl push   bc push   de push   af push
  hl push   bc push
  exx   next,
end-code


\ drop 2drop swap dup ?dup 2dup rot -rot

code drop	\ n --
  hl pop   next,
end-code

code 2drop	\ n1 n2 --
  hl pop   hl pop   next,
end-code

code swap	\ n1 n2 -- n2 n1
  hl pop   (sp), hl ex   hl push   next,
end-code

code 2swap	\ n1 n2 n3 n4 -- n3 n4 n1 n2
  exx   de pop   hl pop   bc pop
  (sp), hl ex   de push   hl push   bc push
  exx   next,
end-code

code dup	\ n -- n n
  hl pop   hl push   hl push   next,
end-code

code ?dup	\ n -- n (n=0); n -- n n (n<>0)
  hl pop   hl push
  a, h ld   a, l or   z, qdupx jr
  hl push
l: qdupx
  next,
end-code

code 2dup	\ n1 n2 -- n1 n2 n1 n2 ; d -- d d
  hl pop   de pop
  de push   hl push   de push   hl push   next,
end-code
\ rot -rot pick roll nip tuck

code rot	\ n1 n2 n3 -- n2 n3 n1
  de pop   hl pop   (sp), hl ex 
  de push   hl push   next,
end-code

code -rot	\ n1 n2 n3 -- n3 n1 n2
  hl pop   de pop   (sp), hl ex
  hl push   de push   next, 
end-code

code pick	\ nn..n0 n -- nn..n0 nn
  hl pop   l sla   h rl   hl, sp add	\ form pointer into stack
  e, (hl) ld   hl inc   d, (hl) ld	\ get nn
  de push   next,			\ return it
end-code

code roll	\ nn..n0 n -- nn-1..n0 nn ; 0 roll does nothing
  hl pop   a, h ld   a, l or   z, rollx jr \ skip if 0
  l sla   h rl   hl, sp add             \ address of demanded
  e, (hl) ld   hl inc                   \ fetch deep in stack
  d, (hl) ld   hl dec                   \
  hl push   exx   hl pop                \ change register sets
  a, a or   hl, sp sbc                  \ form count again
  b, h ld   c, l ld                     \ transfer to bc
  hl, sp add   d, h ld   e, l ld        \ destination in de
  hl dec   de inc                       \ source in hl
  lddr                                  \ transfer bytes
  exx   hl pop   de push                \ replace tos by saved
l: rollx
  next,
end-code

code nip	\ n1 n2 -- n2
  hl pop   de pop   hl push   next,
end-code

code tuck	\ n1 n2 -- n2 n1 n2
  hl pop   de pop
  hl push   de push   hl push   next,
end-code
\ c@ @  c! !  +!  1+ 2+ 1- 2-  2* 2/ u2/

code c@		\ addr -- b ; byte fetch
  hl pop   l, (hl) ld   h, # 0 ld	\ zero high byte of word
  hl push   next,
end-code

code @		\ addr -- n ; word fetch
  hl pop
  e, (hl) ld   hl inc   d, (hl) ld
  de push   next,
end-code
((
code 2@		\ addr -- n1 n2 ; 32 bit fetch - n2 from low address
  hl pop   de, # 3 ld   hl, de add
  d, (hl) ld   hl dec   e, (hl) ld   de push
  hl dec   d, (hl) ld   hl dec   e, (hl) ld   de push
  next,
end-code
))
code c!		\ b addr -- ; byte store
  hl pop   de pop   (hl), e ld   next,
end-code

code !		\ n addr -- ; word store
  hl pop   de pop
  (hl), e ld   hl inc   (hl), d ld   next,
end-code
((
code 2!		\ n1 n2 addr -- ; 32 bit store - n2 at low address
  hl pop
  de pop   (hl), e ld   hl inc   (hl), d ld   hl inc
  de pop   (hl), e ld   hl inc   (hl), d ld   next,
end-code
))
code +!		\ n addr -- ; add n to contents of addr
  hl pop   de pop
  a, (hl) ld   a, e add   (hl), a ld   hl inc
  a, (hl) ld   a, d adc   (hl), a ld   next,
end-code

code 1+		\ n -- n+1
  hl pop   hl inc   hl push   next,
end-code

code 2+		\ n -- n+2
  hl pop   hl inc   hl inc   hl push   next,
end-code

code 1-		\ n -- n-1
  hl pop   hl dec   hl push   next,
end-code

code 2-		\ n -- n-2
  hl pop   hl dec   hl dec   hl push   next,
end-code

code 2*		\ n -- n*2
  hl pop   hl, hl add   hl push   next,
end-code

code 2/		\ n -- n/2 (signed)
  hl pop   h sra   l rr   hl push   next,
end-code

code u2/	\ u -- u/2 (unsigned)
  hl pop   h srl   l rr   hl push   next,
end-code
\ + - d+ d-

code +		\ n1 n2 -- n3
  hl pop   de pop
  hl, de add   hl push   next,
end-code

code -		\ n1 n2 -- n3
  de pop   hl pop 
  a, a or   hl, de sbc   hl push   next,
end-code

code d+		\ d1 d2 -- d3
  de pop  ( ms2 )  exx
  hl pop  ( ls2 )  bc pop  ( ms1 )  de pop ( ls1 )
  hl, de add   hl push   bc push   exx   hl pop
  hl, de adc   hl push   next,
end-code

code d-		\ d1 d2 -- d1-d2
  de pop   exx
  bc pop   de pop   hl pop   a, a or   hl, bc sbc
  hl push   de push   exx  hl pop  hl, de sbc
  hl push   next,
end-code
\ p@ p!                                            sfp 15/04/87

code p@		\ port-addr -- b ; read byte from i/o port
  exx   bc pop   l, (c) in
  h, # 0 ld   hl push   exx   next,
end-code

code p!		\ b port-addr -- ; write byte to i/o port
  exx   bc pop   hl pop   (c), l out   exx   next,
end-code


\ s->d                                    sfp 18/04/87

code s>d	\ n -- d ; convert signed single to double
  hl pop   hl push   h rl   set-t/f jp
end-code


\ within? -- J Lee .. 15/2/94

: within?	\ n1 n2 n3 -- t/f
  >r over >  swap r> >  or 0=	\ is n2 > n1
;				\ is n3 < n1
				\ both must be false

\ on off count bounds                              sfp 18/04/87

code on		\ addr -- ; set word at addr to TRUE
  a, # -1 ld
l: on/off
  hl pop
  (hl), a ld   hl inc   (hl), a ld
  next,
end-code

code off	\ addr -- ; set word at addr to FALSE
  a, a sub   on/off jr
end-code

code count	\ addr -- addr' len ; return start and length of string
   hl pop
   d, # 0 ld   e, (hl) ld			\ length
   hl inc   hl push   de push   next,
end-code

code bounds	\ addr len -- addr+len addr ; return limit and start for DO
  hl pop   de pop   hl, de add
  hl push   de push   next, 
end-code
\ : variable constant user crash defer (;code) does> docreate,

assembler
here is-action-of :
  iy dec   0 (iy), b ld			\ push PI (BC) on return stack
  iy dec   0 (iy), c ld
  hl inc   hl inc   hl inc		\ skip over JP DOCOLON
  c, l ld   b, h ld   next,		\ PFA is new IP
forth

assembler
here is-action-of variable
here is-action-of constant
  hl inc   hl inc   hl inc		\ skip over JP DOVARIABLE
  e, (hl) ld   hl inc   d, (hl) ld	\ get address of data
  de push   next,			\ return that address
forth

assembler
here is-action-of user
  hl inc   hl inc   hl inc		\ skip over JP DOUSER
  e, (hl) ld   hl inc   d, (hl) ld	\ fetch offset
  hl, up ld   hl, de add   hl push	\ add base address from UP
  next,
forth
\ all done 

decimal

;p
