\ Umbilical Forth Target side                      sfp 06/03/88

pto

Jonathan Lee, February 1988
Stephen Pelc, June 1990

Copyright (c) 1988, 1990
MicroProcessor Engineering
133 Hill Lane
Shirley
Southampton SO1 5AF
tel: +44 703 631441
fax: +44 703 339691

October 1991	v2.1 with table driven message passer

\ Send & Receive *** TARGET DEPENDENT ***          Jon 09/06/89

\ If suitable primitives exist use ALIAS to use the suitable primitives
\ as WAIT-BYTE and SEND-BYTE. Otherwise define WAIT-BYTE and SEND-BYTE
\ in a similar fashion to KEY and EMIT respectively.

\ aliased versions
Alias (Key) Wait-Byte
Alias (Emit) Send-Byte

\ non-aliased versions
\ : Wait-byte	\ -- b
\  (key)  ;				\ From serial port to PC host
\
\ : Send-byte	\ b --
\  (Emit)  ;				\ To serial port to PC host

\ wait-ack send&ack                                sfp 12/04/89

: wait-ack	\ -- ; wait for response from host
  wait-byte drop  ;

\ ACK! <>			\ get response and check
 \ IF  message-passer  ENDIF  ;		\   Error!, go restart

: send&ack	\ n -- ; send character, wait for acknowldge
  send-byte				\ send byte
  wait-ack  ;				\ get response and check

\ Send-word  receive-word

Hex

: Send-word	\ n -- ; send 16 bit word to host
  0 0100 um/mod				\ -- bl bh ; -- rem quot
  send&ack  send&ack  ;			\ send both bytes

: Receive-word	\ -- n ; receive 16 bit word from host
  wait-byte  ACK! send-byte		\ high byte
  wait-byte  ACK! send-byte		\ low byte
  swap 0100 * +  ;			\ Convert to a word

decimal


\ depth send-stack receive-stack

: send-stack	\ sys -- ; send stack to host
  depth 0 max				\ get depth, clip underflow
  dup send-word  ?dup			\ tell host what's coming
  if  0 swap 1-				\ massage for -1 +LOOP
      do  i pick send-word  -1 +loop	\ copy it - bottom first
  endif  ;

: receive-stack	\ -- sys ; receive stack from host
  s0 @ sp!				\ clear stack
  receive-word 0			\ get depth
  ?do  receive-word  loop  ;		\ get stack - bottom first
  \ Send-message

hex

l: commands>h  
  ] receive-word [		\ 081 = key
  ] send-word [			\ 082 = emit
  ] receive-word [		\ 083 = push
  ] receive-word [		\ 084 = key?
  ] send-word [			\ 085 = .
  ] receive-stack [		\ 086 = stack to target
  ] send-stack [		\ 087 = stack to host

: Send-message  \ [sys] mess# --
  dup send&ack				\ send command
  081 - dup 6 u>			\ 081..087 -> 0..6
  if  drop message-passer  endif	\  error if out of range
  2* commands>h + @ execute  ;		\ execute selected command


\ tcmd1 tcmd6 tcmd7 tcmd8 tcmd9 ... 

: tcmd1		\ ?? -- ?? ; execute command
  receive-stack				\ receive host stack
  execute  OK! send-byte		\ execute, permit target messages
  send-stack  ;				\ send results back

: tcmd6		\ -- ; fetch operation
  receive-word @ send-word  ;

: tcmd7		\ -- ; store operation
  receive-word receive-word !  ;

: tcmd8		\ -- ; fetch operation
  receive-word c@ send-word  ;

: tcmd9		\ -- ; store operation
  receive-word receive-word c!  ;

: badcmd	\ -- ; bad command
  ERROR! send-byte  message-passer  ;

: tcmd0C	\ -- ; fetch code word
  receive-word @ send-word  ;

\ : tcmd0D	\ -- ; store code word
\  receive-word receive-word !c  ;

: tcmd0E	\ -- ; fetch code byte
  receive-word c@ send-word  ;

\ : tcmd0F	\ -- ; store code byte
\  receive-word receive-word c!c  ;

\ 25/11/91 sfp; commands>t message-passer run-umbilical
  
l: commands>t
  ] tcmd1 [		\ 1 = execute
  ] receive-word [	\ 2 = push
  ] send-word [		\ 3 = pop
  ] receive-stack [	\ 4 = receive stack
  ] send-stack [	\ 5 = send stack
  ] tcmd6 [		\ 6 = fetch - @
  ] tcmd7 [		\ 7 = store - !
  ] tcmd8 [		\ 8 = c fetch - C@
  ] tcmd9 [		\ 9 = c store - C!
  ] badcmd [		\ 0A, 0B = L@,L! for 32 bits
  ] badcmd [
  ] tcmd0C [		\ 0C = fetch code word - @
  ] badcmd ( tcmd0D ) [	\ 0D = store code word - !C
  ] tcmd0E [		\ 0E = fetch code byte - C@
  ] badcmd ( tcmd0F ) [	\ 0F = store code byte - C!C
  ] badcmd [		\ 010, 011 = L@, L!C for 32 bits
  ] badcmd [

hex


: depth		\ -- n ; return no. of items on stack before DEPTH
  s0 @ sp@ - 2- 2/  ;

: Message-passer	\ -- ; Umbilical Forth message loop
  r0 @ rp!				\ no exit from here
  BEGIN
    wait-byte				\ get command byte
    1- dup 010 u>			\ if invalid command (range 1..011)
    if  ERROR!				\ shout at host
    else  ACK! send-byte		\ acknowledge valid command
          2* commands>t + @ execute	\ execute selected command
          wait-ack			\ wait for host acknowledgement
          depth max-depth u>		\ if stack undeflow
          if  s0 @ sp!  SUFL!		\ reset stack and tell host
          else  OK!			\ else say OK
          endif
    endif
    send-byte
  AGAIN  ;

decimal

: run-umbilical	\ -- ; run umbilical forth
  r> drop				\ no exit
  (init)				\ Forth initialisation
  init-ser				\ initialise serial line
  message-passer  ;			\ run message passer

