\ 		    MODULAR FORTH EXPORT MANAGER V1.0
\
\ Author: Rod Crawford
\
\ Written: 23/2/89
\
\ Mods: NONE
\
\ Desc: This package permits more than the standard 64 words to
\ 	be exported by any one module in a Modular Forth Application.
\ 	Up to 256 wors may now be exported. If more are required
\ 	the constant #Exports in this file should be changed.
\
\
\ Generating and Export List:
\
\ 	All pages in this file should be compiled into the
\ 	module from which items are to be exported.
\ 	A list of the words to be exported should then be loaded.
\ 	This list is headed with the term:
\
\ 	     $filename EXPORTS{
\
\ 	Terminated with the term:
\
\ 	    }EXPORTS
\
\ 	Each item in the list has the form.
\
\ 	    EXPORT <wordname>
\
\ 	For Example if we wished to export the two words FOO and BAR the
\ 	export list would be:
\
\ 	    $filename EXPORTS{
\ 			 EXPORT FOO
\ 			 EXPORT BAR
\ 	    }EXPORTS
\
\ 	The export mananger builds an export list in the module
\ 	and generates an Import List in a file (IMPORTS.LST).
\
\
\ Importing Words
\
\ 	Modules which wish to import words should compile the
\ 	definition of IMPORT from the file IMPORT.4th
\
\ 	The Import List should then be loaded. From the file
\ 	IMPORTS.LST
\ 	Each item in the file has the form:
\
\ 	    n IMPORT <wordname>
\

Only Forth				Vocabulary Exporting
Only Forth Also Exporting	      Exporting Definitions

\ CONSTANTS
\

2 Constant /Export
256 Constant #Exports

\ VARIABLES

Variable Dispatch#

PCB Imports
Variable Terminal   \ The handle for terminal output

\ Export Table Definition.

Create Exports	    #Exports /Export * Allot


\ *****************************  Th-Export  ******************************
\
\ Args: Index		Index of the Export required
\
\ Rets: Export		Address of the exported word in the Export Table
\
\ Desc: This word indexes the export table.
\

: Th-Export \ Index -- Export
    /Export * Exports + ;


\ *****************************  Exports{  ********************************
\
\ Args: NONE
\
\ Rets: NONE
\
\ Desc: Opens the imports file, moves and defines
\ 	all output to go to the file.
\

Only Forth Also Exporting Also Forth  Forth Definitions

: Exports{ \ $addr -- ; $addr is the filename
    Op-Handle @ Terminal !
    Imports Set-Pathname
    Imports Create-Path-PCB    Abort" Can't Open Import File"
    Imports Handle Op-Handle ! ;

\ *****************************  }Exports  ********************************
\
\ Args: NONE
\
\ Rets: NONE
\
\ Desc: Closes the imports file. And ensures all output is
\ 	directed to the the previous output stream.
\

: }Exports \ --
    Terminal @ Op-Handle !
    Imports Handle Close-Path	Abort" Can't Close Import File" ;




\ *****************************  Dispatch  ********************************
\
\ Args: Index	    Index of the word to be dispatrched.
\
\ Rets: NONE
\
\ Desc: Given the index of an entry in the export table, this word
\ 	causes it to be executed.
\ 	Dispach is made public so that it can be called from other
\ 	modules by words which are children of Import. It should NOT
\ 	be called by user application programs.


: Dispatch \ Index --
    Th-Export @ Execute ;  PUBLIC

Only Forth Also Exporting	      Exporting Definitions


\ ***************************  .Import	***********************************
\
\ Args: Name	    String representing the name of the word that
\ 		    will be Imported.
\
\ Rets: NONE
\
\ Desc: Given the name of a word which can be imported by an another
\ 	module, generates an entry in the imports file for it.
\ 	The entry consist of the index of the word in the Export
\ 	table, the word IMPORT and the word name, i.e.
\ 	    nn IMPORT <wordname>
\
: .Import  \ --
    Dispatch# ?  ." IMPORT " $. CR ;


\ ***************************  Export  ***********************************
\
\ Args: NONE	; Wordname in input
\
\ Rets: NONE
\
\ Desc: Adds the given word in the input to the export table.
\ 	An entry for the word is also made in the Import file.
\
Only Forth Also Exporting Also Forth	Forth Definitions

: Export \ -- ; Word in input.
    Dispatch# @ #Exports = Abort" Export Table Full!"
    Here ' Dispatch# @ Th-Export !
    .Import   1 Dispatch# +! ;
