\ Serial i/o drivers for Z80/64180 + DART + CTC interrupt driven

((
Copyright (c) 1991

MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
England

tel: (+44) 703 631441
fax: (+44) 703 339691

First Release:	May 1991

Changes:	See RELEASE.DOC for full details
))
\ DART/CTC interrupt driven                        sfp 26/03/90

only forth definitions

hex
\
\ Define address used as base of DART interrupt vectors.
\ 16 bytes are needed for the 8 vectors.
\
int-page 010 + equ dart-vector

\
\ DART - i/o addressing & bit masks                sfp 26/03/90
\ set up for GNC Electronics Cardmaster CPU
\
020		equ dart		\ base address of Z80 DART chip
dart            equ a-data              \ CH A is RS422
dart 1+         equ b-data              \ CH B is RS232
dart 2+         equ a-cmd
dart 3 +        equ b-cmd
01              equ rda                 \ data available bit
04              equ tbe                 \ transmit buffer bit

030             equ ctc                 \ base address of Z80 CTC chip
ctc             equ ctc0                \ CH 0 for RS422
ctc 1+          equ ctc1                \ CH 1 for RS232
ctc 2+          equ ctc2                \ CH 2 for single step
ctc 3 +         equ ctc3                \ CH 3 for MCP

assembler
l: init-ctc
  a, # 057 ld   ctc1 a out              \ initialse CTC1
  a, # 0D ld   ctc1 a out               \ set CTC1 to 9600 baud
  ret
forth

assembler
l: init-dart
  a, # 018 ld   b-cmd a out             \ reset DART channel B
  a, # 2 ld   b-cmd a out               \ select WR2
  a, # dart-vector .lo ld   b-cmd a out \ set interrupt vector
  a, # 4 ld   b-cmd a out   a, # 044 ld   b-cmd a out
  a, # 3 ld   b-cmd a out   a, # 0C1 ld   b-cmd a out
  a, # 5 ld   b-cmd a out   a, # 068 ld   b-cmd a out
  a, # 1 ld   b-cmd a out   a, # 014 ld   b-cmd a out
  ret
forth

code init-ser	\ -- ; initialise serial
  di                                    \ no interrupts
  init-ctc call                         \ deal with CTC
  init-dart call                        \ deal with DART
  2 im                                  \ interrupt mode 2
  a, # dart-vector .hi ld   i, a ld     \ set vector page
  ei                                    \ enable mode 2 ints
  next,					\ all done
end-code
\ DART - (emit) emit cr ................           sfp 26/03/90

hex

code (emit)	\ char --
l: char-op
  a, # 0 ld   b-cmd a out   nop         \ select reg
  a, b-cmd in   a, # tbe and   z, char-op jr \ spin until ready
  hl pop   a, l ld   b-data a out       \ transmit character
  next,
end-code

: emit		\ char --
  (emit)  1 out +!  ;

: type		\ addr len --
  bounds ?do  i c@ emit  loop  ;

: cr		\ --
  0A emit  0D emit  out off  ;

\
\ These variables are used by the interrupt routines to communicate
\ with the main program.
\
variable rx-avail               \ character available
variable rx-char                \ character read
variable rx-error               \ last error

assembler
l: rx-int       \ main receiver interrupt
  af push
  a, b-data in   rx-char a ld   \ read and save character
  a, # -1 ld   rx-avail a ld    \ set flag
  af pop   ei   reti
forth

assembler
l: src-int      \ entered after an error
  af push
  a, # 01 ld   b-cmd a out   nop        \ select reg 1
  a, b-cmd in   rx-error a ld           \ read it for error type
  a, # 030 ld   b-cmd a out             \ issue error reset
  a, b-data in   rx-char a ld           \ read & save character
  a, # 0FF ld   rx-avail a ld           \ set flag
  af pop   ei   reti
forth

rx-int dart-vector 4 + !		\ receiver interrupt
src-int dart-vector 6 + !		\ error interrupt
\ DART - (key) key .....................           sfp 28/03/90

hex

code (key)      \ -- char ; put processor to sleep while no character ready
  begin,  a, rx-avail ld   a, a or      \ test flag
    z, while,  halt                     \ stop while no flag
  repeat,
  h, # 0 ld   a, rx-char ld   l, a ld   \ form character
  a, # 0 ld   rx-avail a ld             \ reset flag
  hl push   next,			\ return character
end-code

code (key?)     \ -- t/f ; return true if charater ready
  a, rx-avail ld   h, a ld   l, a ld    \ get flag
  hl push   next,   end-code            \ return it

: key?		\ -- t/f
  (key?)  ;
\ all done

decimal

;p
