\ Testing for Forth words
\
\ Adapted from 'Testing Toolkit' by P. Koopman Jr. - Forth Dimensions XII,3
\
\ J Lee :- 5/10/90
\
\

\
\ This wordset provides a means of testing programs in a formalised fashion.
\ A word is executed surrounded by a formal stack definition.
\
\ DS( specified_inputs --) <WORD> (-- required_outputs )DS
\


\ Try Ds( --) (-- )Ds .. J Lee -- 05/10/90
\

variable Stack1		\ stack depth marker
variable Stack2		\   "	"	  "
variable Stack3		\   "	"	  "

variable Test-Word	\ CFA of word under test

variable Left-Loop	\ flag for early loop exit


hex

: Try		( -- <wordname> )
	' dup test-word !
	execute  ;

: Ds(		( -- sentinel )
	0bad1				\ sentinel (used to check for stack corruption)
	depth stack1 !  ;		\ intial depth

: --)		( .. -- .. )
	depth stack2 !  		\ pre execution depth
	try  ;				\ execute word

: (--		( .. -- .. )
	depth stack3 !  ;		\ post execution depth

: Reset-Stack	( -- ; reset data stack )
	s0 @ sp!  ;
\ continued

: Check-Element	( element# -- t/f )
	dup						\ only do for non-zero elements
	If
		roll <>
		If
			." FAILED, Incorrect Result"
			reset-stack
			Left-Loop on			\ flag that loop is left prematurely
			-1					\ leave TRUE
		Else
			0					\ leave FALSE
		Endif
	Endif  ;

: Check-Stack	( sentinel -- ; check stack for corruption )
	0bad1 <>					\ check stack corruption
	If
		." FAILED, stack corrupted"
		s0 @ sp!				\ reset data stack
	Else
		." PASSED."
	Endif  ;

: )Ds
	cr ." Testing: " test-word @ >name .name ."  :- "
	depth stack3 @ - dup	\ post execution elements
	stack3 @ stack1 @ -		\ pre execution elements
	<>					\ give up if not the same
	If
		." FAILED, Number of elements not the same"
		reset-stack
	Else
		left-loop off					\ turn loop exit flag off
		0 swap
		?Do					    		\ check result against expected
			i check-element ?leave
			-1
		+Loop
		left-loop @ 0=
		If  check-stack  Endif
	Endif  ;

decimal
\ Test it out .. J Lee -- 05/10/90

\
\ A word is executed surrounded by a formal stack definition.
\
\ DS( specified_inputs --) <WORD> (-- required_outputs )DS
\

decimal

ds( --) noop (-- )ds

ds( 1234 --) drop (-- )ds

ds( 1234 --) dup (-- 1234 1234 )ds

ds( 1234 5678 --) swap (-- 5678 1234 )ds

ds( 1 2 3 --) rot (-- 2 3 1 )ds

ds( 1 2 --) nip (-- 2 )ds

ds( 1111 4444 --) + (-- 5555 )ds

ds( 4444 1111 --) - (-- 3333 )ds

ds( 10000 5 --) / (-- 2000 )ds

ds( 200 5 --) * (-- 1000 )ds
\ all done

decimal


