\ Z80/64180 ROM PowerForth - Interrupts
pto

Copyright (c) 1988-1991 to:

MicroProcessor Engineering Limited
133 Hill Lane
Shirley
Southampton SO1 5AF
England

tel: (+44) 703 631441   ( international )
tel: 0703 631441        ( domestic )

May 1991	Code separated from multitasker code

\ common-isr

decimal

\
\ This common code is entered by all high level interrupt calls.
\ COMMON-ISR saves all the registers not saved at the entry point,
\ and sets up the user area pointer UP to point to the interrupt
\ page.
\
assembler
l: common-isr                           \ common for all ints
  de push   hl push                     \ save regs, AF/BC done
  af, af' ex   exx                      \ swap register banks
  af push   bc push   de push   hl push \ save registers
  af, af' ex   exx                      \ swap back
  ix push   iy push                     \ save index registers
  hl, up ld   hl push                   \ save user area pointer
  hl, # int-init-u0 ld   up hl ld	\   and reset it
  ix, # next ld   next,                 \ set IX, and go
forth


\ reti

code reti	\ -- ; return from int.
  hl pop   up hl ld                     \ restore user pointer
  iy pop   ix pop                       \ restore index regs
  af, af' ex   exx                      \ restore alternate reg
  hl pop   de pop   bc pop   af pop
  af, af' ex   exx                      \ restore main regs
  hl pop   de pop   bc pop   af pop
\
\ depending on the interrupt mode in use (may be mode 2)
\ the phrase  "ei ret" may have to be replaced by "ei reti"
\
  ei  reti				\ get back (mode 2)
end-code


\ di ei

code di		\ -- ; disable interrupts
  di   next,
end-code

code ei		\ -- ; enable interrupts
  ei   next, 
end-code

code save-int		\ -- n ; save interrupt status, and disable interrupts
  a, i ld   af push			\ after LD A,I status P is IFF2
  di  next,				\ then disable interrupts
end-code

code restore-int	\ n -- ; restore interrupt status
  af pop				\ get status back
  pe, if,  ei  else,  di  endif,	\ reset as required
  next,
end-code

\ example interrupt handler for 64180
pto

hex		

variable counter	\ -- addr ; holds count of interrupts
	 
: reset-timer		\ --
  TCR p@			\ read TCR
  TMDR0L p@			\ read LSB TMDR
  2drop				\ don't care about results
;			    

: start-timer		\ --		       
  011 TCR p!	     		\ enable downcount ints on TMR 0
  ei				\ ensure IEF1 enabled
;  	 

: stop-timer
  000 TCR p!			\ disable downcount interrupt
;

: example-isr		\ -- ; stack must be balanced ; just update counter
  reset-timer				\ clear int source
  1 counter +!				\ timer is automatically reset
; 					\ to reload value
		
defer timer0-isr

l: timer0-int		\ code to be executed is compiled here
  ] timer0-isr reti [
  
assembler
l: timer0-int0		\ takes 8 bytes, so can be assembled in RST x vector
  af push   bc push			\ save status and IP
  bc, # timer0-int ld			\ point to Forth code to run
  common-isr jp				\ join common code
forth

timer0-int0  int-page int-page-l + 4 +  !  \ set mode 2 vector for timer 0
					\ Note INT-PAGE-L defined in START180

: run-example		  \ --
  assign example-isr to-do timer0-isr	\ patch vector
  start-timer				\ go
;

\ all done

decimal

;p
