\ Local variables for Cross Compiler - Target code: Z80

pto

file: LOCALS-T.FTH

Copyright (c) 1994 MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 1703 631441
fax: +44 1703 339691
net: mpe@mpeltd.demon.co.uk

May 1994	Original release
November 1994	Separated into Separate host and target parts for XC.
		Ported to Z80 family

This file implements the following syntax for named inputs and local
variables. The sequence
  { ni1 ni2 ... | lv1 lv2 ... -- o1 o2 }
defines named inputs, local variables, and outputs. The named inputs are
automatically copied from the data stack on entry. Named inputs and local
variables can be referenced by name within the word during compilation.
The output names are dummies to allow a complete stack comment to be
generated.

The items between { and | are named inputs.
The items between | and -- are local variables.
The items between -- and } are outputs.

Named inputs and locals return their values when referenced, and must
be preceded by -> to perform a store, or by ADDR to return the address.

In the example below, a and b are named inputs, and a+b and a*b are local
variables.

: foo		{ a b | a+b a*b -- }
  a b + -> a+b
  a b * -> a*b
  cr a+b .  a*b .
;

In this implementation, 512 bytes of dictionary space are allocated for
local variables, allowing up to about 100 locals, allowing for nesting.
Local variables are only visible in the word being compiled. Named inputs
and local variables may only be single cell values.


\ the local area

only forth definitions
decimal

((
   ^local-base : n cells : ...
   ^                       ^    
   | 		    	   |
   local-base		   local-top
))


#256 constant #locals-cells	\ number of cells per local stack

\ Each task must have its own locals stack, this is the one for task 0
there constant locals-stack0	\ -- addr
    #locals-cells cell* allot-ram

variable local-base		\ base of current locals stack frame
variable local-top		\ top of locals stack


: reset-locals	\ locals-stack -- ; initialise locals
  dup local-base !
  local-top !
;

((
: +locals	\ n -- ; allocates space for n bytes
  local-base @				\ save
  local-top @ local-base !		\ set new base
  swap cell+ local-base @ +		\ add requested cells + 1
  local-top !				\ to form new top
  local-base @ !			\ save pointer to old local-base
;
))

\ *local-top := local-base
\ local-base := local-top
\ local-top := local-top + #bytes
\
\ Z80 family
code +locals	\ -- ; allocates space for b bytes, b is an inline byte
  h, # 0 ld   a, (bc) ld  l, a ld	\ get inline #bytes to allocate
  bc inc  bc push			\ step IP on, save IP
  hl push				\ preserve #bytes to allocate

\ *local-top := local-base
  de, local-base ld			\ local-base@
  hl, local-top ld			\ local-top@
  (hl), e ld  hl inc  (hl), d ld	\ !

\ local-base := local-top
  de, local-top ld			\ local-top@
  hl, # local-base ld			\ local-base
  (hl), e ld  hl inc  (hl), d ld	\ !

\ local-top := local-top + #bytes
  hl, local-top ld			\ local-top@
  de pop  hl, de add			\ #bytes +
  de, hl ex				\
  hl, # local-top ld			\ local-top
  (hl), e ld  hl inc  (hl), d ld	\ !

  bc pop				\ restore IP
  next,
end-code


(( - Intel 386
code +locals	\ -- ; allocates space for n bytes, n found inline
  cli					\ indivisible
  mov edx, local-base [edi]		\ save old local base
  mov ecx, local-top [edi]   mov local-base [edi] , ecx	\ set new lbase
  lodsd   add eax, ecx   mov local-top [edi] , eax	\ ltop=base+n
  mov 0 [ecx] [edi] , edx
  sti
  next,
end-code
))

((
: -locals	\ -- ; step back to previous frame
  local-base @ @			\ get previous frame
  local-base @ local-top !		\ step back
  local-base !
;
))

\ local-top := local-base
\ local-base := *local-base
\
\ Z80 family
code -locals	\ -- ; step back to previous frame
  de, local-base ld			\ local-base @
  hl, # local-top ld			\ local-top
  (hl), e ld  hl inc  (hl), d ld	\ !

  hl, local-base ld			\ local-base @
  e, (hl) ld  hl inc  d, (hl) ld	\ @
  hl, # local-base ld			\ local-base
  (hl), e ld  hl inc  (hl), d ld	\ !

  next,
end-code

(( - Intel x86
code -locals	\ -- ; step back to previous frame
  cli					\ indivisible
  mov eax, local-base [edi]   mov ecx, 0 [eax] [edi]	\ get previous frame
  mov local-top [edi] , eax		\ step back
  mov local-base [edi] , ecx
  sti
  next,
end-code
))

((
: .locals	\ -- ; display locals pointers
  base @ hex
  cr ." Local Frames : " local-frames u.
  cr ." Local-Base   : " local-base @ u.
  cr ." Local-Top    : " local-top @ u.
  cr ." Frame size   : " local-top @ local-base @ - dup u.
  ?dup
  if  ."  Room for " cell u/ 1- u. ." cells"  endif
  base !
;
))

\ `TO' variable implementation : runtime

((
the following syntax is used
VAR JIM
  JIM returns the value		JIM .
  -> JIM stores a value		5 -> JIM
  addr JIM returns the address  addr JIM @

The operators set a variable OP-TYPE at compile time to indicate which
operation is required. The VARS or other users of this technique are
state-smart and process the variable at compile time.
))

\ Z80 family
code @lv	\ -- n ; fetch value of local variable, offset byte in-line
  h, # 0 ld   a, (bc) ld  l, a ld	\ get inline #byte to reference
  bc inc		       		\ step IP on
  de, local-base ld			\ local-base @
  hl, de add				\ #bytes +
  e, (hl) ld  hl inc  d, (hl) ld	\ @
  de push				\
  next,
end-code

(( - Intel x86
code @lv	\ -- n ; fetch value of local variable, offset in-line
  push ebx			\ save tos
  lodsd				\ get offset
  add eax, local-base [edi]	\ add base
  mov ebx, 0 [eax] [edi]	\ get value
  next,
end-code
))

\ Z80 family
code !lv	\ n -- ; store into local, offset in-line
  h, # 0 ld   a, (bc) ld  l, a ld	\ get inline #byte to reference
  bc inc		       		\ step IP on
  de, local-base ld			\ local-base @
  hl, de add				\ #bytes +
  de pop				\
  (hl), e ld  hl inc  (hl), d ld	\ !
  next,
end-code

(( - Intel x86
code !lv	\ n -- ; store into local, offset in-line
  lodsd				\ get offset
  add eax, local-base [edi]	\ add base
  mov 0 [eax] [edi] , ebx	\ store value
  pop ebx			\ restore TOS
  next,
end-code
))

\ Z80 family
code lv		\ -- addr ; fetch address of local variable, offset in-line
  h, # 0 ld   a, (bc) ld  l, a ld	\ get inline #byte to reference
  bc inc		       		\ step IP on
  de, local-base ld			\ local-base @
  hl, de add				\ #bytes +
  hl push				\
  next,
end-code

(( - Intel x86
code lv		\ -- addr ; fetch address of local variable, offset in-line
  push ebx			\ save tos
  lodsd				\ get offset
  add eax, local-base [edi]	\ add base
  mov ebx, eax			\ get address
  next,
end-code
))

\ test words

((
\ Each task must initialise its own locals frame stack
: Task1  ( -- )
  locals-stack1 reset-locals	\ initialise locals stack
  Begin				\
    job				\ task code
    pause			\
  Again				\
;
))

((
: t2	{ a b -- }
  cr ." a=" a .  ."  b=" b .
  cr ." a+b=" a b + .
;

: t3	{ a b | a+b a*b -- }
  a b + -> a+b
  a b * -> a*b
  a+b .  a*b .
;
))