\ Z80/64180 ROM PowerForth - Multi-Tasker & Int      sfp 09/05/91

((
Copyright (c) 1988-1991 to:

MicroProcessor Engineering Limited
133 Hill Lane
Shirley
Southampton SO1 5AF
England

tel: (+44) 703 631441   ( international )
tel: 0703 631441        ( domestic )

September 1988: dynamic task reallocation and INIT-TASK added
October 1988  : page switching bullet-proofed, SELF returns tsk#
February 1990 : INIT-TCBS & INIT-MULTI refactored
November 1994 : Amended for Local variable support: PAUSE, ACTIVATE
))\ define TCB area offsets

decimal

\
\ offsets within the TCB save area
\
  0 equ tcbsp                   \ task's SP
  2 equ tcbst                   \ task status byte, bit = 1 means:
				\ bit 7 = task running
				\ bit 6 = message present
				\ bit 5 = event triggered but not run
				\ bit 4 = task restarted by event
				\ bit 3..0 = user defines these
  3 equ tcbid                   \ id of message sender
  4 equ tcbmsg                  \ task message data
  6 equ tcbaction               \ CFA of main word
  8 equ tcbevent                \ CFA of event word

  10 equ tcblen                 \ length of task control block
\
\ N.B. If TCBLEN is changed the subroutine TASK>ADDR must be rewritten

  7 equ runbit			\ see above for details
  6 equ msgbit
  5 equ evtrigbit
  4 equ evrunbit

\ reserve RAM space for variables and TCBS
hex
variable curtask		\ holds pointer to TCB of current task
variable task#			\ holds task no. of current task
  080 task# c!			\ Top bit set indicates tasker disabled

there constant tcbs
  tcblen #tasks * allot-ram     \ reserve TCB space
there equ endtcbs


\ pause

hex

code pause	\ -- ; the scheduler
L: [schedule]
  a, task# ld   a, # 080 and		\ test running bit of TASK#
  nz, xschedule jp			\ skip scheduler if not running
\
\ Save previously running task's IX, IP, RP, UP, SP.
\ If you need an interrupt driven scheduler save ALL the other registers
\ as well here.
\

\
\ if you change the number of items saved you must also modify ACTIVATE
\ below, but life will be easier if IX, IP, RP, and UP are the last items
\ pushed, and in that order, as the equate #STACK-ITEMS/TASK will adjust
\ stack depths for you.
\
6 equ #stack-items/task 	\ *jl* 18/11/94 - changed for locals

  ix push				\ just in case
  bc push				\ save IP
  iy push				\ save RP
  hl, up ld   hl push			\ save UP

\ *jl* 18/11/94 - added for locals
  hl, local-base ld  hl push		\ save local-base
  hl, local-top ld  hl push		\ save local-top

  hl, # 0 ld  hl, sp add  		\ save SP in TCB
  ix, curtask ld			\  point to TCB
  tcbsp (ix), l ld  tcbsp 1+ (ix), h ld	\  store stack pinter
\
\ select next task to run
\
  begin,
    de, # tcblen ld   ix, de add	\ step task TCB pointer
    a, task# ld   a inc   task# a ld	\ step task I.D.
    a, # #tasks cp			\ at end of TCBs ?
    z, if,  ix, # tcbs ld		\ yes, reset TCB pointer
	    a, a xor   task# a ld	\      and TASK#
       endif,
    a, tcbst (ix) ld   a, a or		\ running if TCBST.7 set
  nz, until,				\ check task is running
\
\ run selected task - sp, up, rp, ip
\
  curtask ix ld				\ set as next task
  l, tcbsp (ix) ld  h, tcbsp 1+ (ix) ld	\ restore SP from TCB
  sp, hl ld				\

\ *jl* 18/11/94 - added for locals
  hl pop  local-top hl ld		\ restore local-top
  hl pop  local-base hl ld		\ restore local-base

  hl pop    up hl ld			\ restore UP
  iy pop				\ restore RP
  bc pop				\ restore IP
\
\ For an interrupt driven scheduler pop all the other registers here.
\

\
\ Now test if the event trigger flag for this task is set, and run
\ the handler if this bit is set.
\
  evtrigbit tcbst (ix) bit		\ test event triggered bit, set?
  nz, if,   evtrigbit tcbst (ix) res	\ y:, reset triggered bit
	    evrunbit tcbst (ix) set	\     set event run bit
  	    l, tcbevent (ix) ld		\     HL=CFA of word to run
	    h, tcbevent 1+ (ix) ld	\
	    ix pop			\ restore IX
	    (hl) jp			\ go execute word
  endif,
  ix pop				\ restore IX
l: xschedule
  next,					\ back to Forth
end-code
\ taskaddr task>addr self status wait-event/msg wait stop

\
\ The subroutine TASK>ADDR is used to convert a task number into the
\ address of its TCB. It will have to be modified whenever TCBLEN is changed.
\ At the time of writing TCBLEN is ten bytes. The conditional compilation check
\ below tests this assumtion. The subroutine also assumes that #TASKS * TCBLEN
\ is never greater than 256 bytes, so making the calculation faster.
\
decimal
tcblen 10 <>
if(  cr ." TCBLEN mismatch"   -1 error  )endif

code taskaddr	\ task# -- addr ; return address of task's TCB
  hl pop				\ get task number
  ix push   task>addr call   ix pop	\ convert (see below)
  hl push   next,			\ return address

l: task>addr	\ converts task number in L to address in IX and HL
  a, l ld   a, l add			\ A=2*task
  a, a add   a, a add			\ A=8*task
  a, l add   a, l add			\ A=10*task
  h, # 0 ld   l, a ld			\ HL=10*task
  de, # tcbs ld   hl, de add		\ HL=TCBS + 10*task
  hl push   ix pop			\ IX=TCBS + 10*task
  ret
end-code

hex

code self       \ -- task# ; push current task#
  a, task# ld   a, # 07F and		\ fetch and mask TASK#
  l, a ld   h, # 0 ld			\ form 16 bit number
  hl push   next,			\ return it
end-code

code status	\ -- status ; return task's status, but with running bit
		\ masked so it is only set by a message, event, or user
  ix push   ix, curtask ld		\ point to own TCB
  a, tcbst (ix) ld   a, # 07F and	\ fetch and mask
  h, # 0 ld   l, a ld   hl push		\ return status byte
  next,
end-code

: wait-event/msg        \ -- ; wait for message or event trigger
  begin  pause  status  until  ;	\ wait for event, message, or user

: wait		\ delay -- ; 0=for ever, n=wait n cycles
  ?dup					\ 0 case is special
  if  0					\ wait n cycles
      do  pause  loop
  else  wait-event/msg			\ wait until woken
  endif  ;

: stop          \ -- ; halts current task
  self halt  pause  ;
\ single multi restart halt set-event clr-event-run ?event msg?

hex

code single     \ -- ; disable scheduler by setting flag
  a, task# ld  a, # 080 or  task# a ld	\ set bit in TASK#
  next,
end-code

code multi      \ -- ; enable  scheduler by clearing flag
  a, task# ld  a, # 07F and  task# a ld	\ set bit in TASK#
  next,
end-code

code restart    \ task# -- ; mark task TCB as running
  hl pop				\ get task#
  ix push   task>addr call		\ task# in IX
  runbit tcbst (ix) set			\ set running bit in status byte
  ix pop   next,			\ all done
end-code

code halt       \ task# -- ; reset running bit in TCB
  hl pop				\ get task#
  ix push   task>addr call		\ task# in IX
  runbit tcbst (ix) res			\ set running bit in status byte
  ix pop   next,			\ all done
end-code

code set-event	\ task# -- ; set task's event flag - used for testing
  hl pop				\ get task#
  ix push   task>addr call		\ task# in IX
  evtrigbit tcbst (ix) set		\ set running bit in status byte
  ix pop   next,			\ all done
end-code

code clr-event-run	\ -- ; clear own task's event-run flag
  ix push				\ might be used by Z80
  ix, curtask ld   task>addr call	\ task address in IX
  evrunbit tcbst (ix) res		\ set running bit in status byte
  ix pop   next,			\ all done
end-code

code ?event     \ -- ; run event if flag set (own task only)
  ix push   ix, curtask ld		\ point to current task
  evtrigbit tcbst (ix) bit		\ test event triggered bit, set?
  nz, if,   evtrigbit tcbst (ix) res	\ y:, reset triggered bit
	    evrunbit tcbst (ix) set	\     set event run bit
            l, tcbevent (ix) ld		\     HL=CFA of word to run
            h, tcbevent 1+ (ix) ld	\
	    ix pop			\ restore IX
	    (hl) jp			\ go execute word
  endif,
  ix pop				\ restore IX
  next,
end-code
\ msg? event? (send-message) (get-message)

code msg?       \ task# -- flag ; true if task has message
  hl pop				\ get task#
  ix push   task>addr call		\ task# in IX
  hl, # 0 ld				\ assume no message
  msgbit tcbst (ix) bit			\ test message bit in status byte
  nz, if,  hl dec  endif,		\ bad assumption, correct it
  ix pop				\ restore IX
  hl push   next,			\ all done
end-code

code event?     \ task# -- flag ; true if task had event
  hl pop				\ get task#
  ix push   task>addr call		\ task# in IX
  hl, # 0 ld				\ assume no message
  evrunbit tcbst (ix) bit		\ test event run bit in status byte
  nz, if,  hl dec  endif,		\ bad assumption, correct it
  ix pop				\ restore IX
  hl push   next,			\ all done
end-code

code (send-msg) \ addr task# -- ; store message in task
  hl pop   exx   de pop   exx		\ get task# and CFA
  ix push   task>addr call		\ task# in IX
  exx   tcbmsg (ix), e ld		\ store message in task
  tcbmsg 1+ (ix), d ld   exx
  a, task# ld   a, # 07F and		\ fetch and mask task number
  tcbid (ix), a ld			\  into TCB
  msgbit tcbst (ix) set			\ set message bit in task
  ix pop   				\ restore IX again
  next,
end-code

code (get-message)      \ -- addr task# ; fetch message and sending task
  ix push   ix, curtask ld		\ point to current task
  l, tcbmsg (ix) ld			\ fetch 16 bit message
  h, tcbmsg 1+ (ix) ld
  e, tcbid (ix) ld   d, # 0 ld		\ fetch 8 bit task ID
  msgbit tcbst (ix) res			\ reset task's message bit
  ix pop				\ restore IX
  hl push   de push			\ return address (hl) and task (de)
  next,
end-code
\ send-message get-message to-task to-event

: send-message  \ addr task# -- ; send message to task
  begin  dup msg?               \ if task already has a message
    while  pause                \ wait
  repeat                        \ and try again
  (send-msg)  ;                 \ send the message

: get-message   \ -- addr task# ; wait for any message
  begin  self msg? 0=           \ check for a pending message
    while  pause                \ wait for it
  repeat                        \ try again
  (get-message)  ;              \ read it

: to-task	\ cfa task# -- ; sets action of task when activated
  tcblen * tcbs + tcbaction + !  ;      \ set CFA into TCB of task

: to-event	\ cfa event# -- ; sets action of event handler when triggered
  tcblen * tcbs + tcbevent + !  ;       \ set CFA into TCB of task
\ activate

\
\ The ACTIVATE function initialises a tasks's TCB, generates an initial stack
\ for that task, and then marks it as running.
\

\ The equate #STACK-ITEMS/TASK defines how many items are saved on a task's
\ data stack by the scheduler. The equate INIT-STACK-DEPTH gives the
\ initial stack depth in bytes.
\
#stack-items/task 2* equ init-stack-depth

\
\ The last things saved on the stack by the scheduler are IX, IP, RP, and UP,
\ which are pushed in that order.
\ *jl* 18/11/94 added local-base, local top to above list
\
: activate      \ task# -- ; start task from scratch
  dup tcblen * tcbs + >r                \ TCB addr saved
  per-task *				\ offset of user area from INIT-U0
  init-s0 over +  init-stack-depth -	\ form initial SP with 4 items
  dup r@ tcbsp + !			\  and set task's SP
\ -- task# offset task-sp

\ *jl* 18/11/94 added for locals
  0 over !				\ local-top - must be set by task
  0 over 2+ !				\ local-base - must be set by task
\ *jl* 18/11/94 modified offsets because of locals
  over init-u0 +  over 4 + !            \ put UP at SP+4
  over init-r0 +  over 6 + !            \ put RP at SP+6
  r@ tcbaction +  over 8 + !            \ put IP at SP+8
  next over 10 + !			\ put IX at SP+10

  2drop					\ discard task-sp and offset
  080 r> tcbst + c!  ;			\ initial value of status byte in TCB

: init-multi    \ -- ; initialisation with multi-tasking
  single				\ just in case!
\
\ first set the task user areas to a well defined state
\
  [ init-u0 per-task + ] literal        \ skip task 0
  [ #tasks per-task * ] literal bounds  \  but include interrupt task area
  do  i per-task erase                  \ clear task space
      init-u0 i 040 cmove               \ reset user areas
  per-task +loop                        \ in each task
\
\ then initialise the task control blocks, only task 0 needs to be set up.
\
  tcbs  endtcbs over -  erase		\ clear TCB area
  080 tcbs tcbst + c!			\ set status of task 0 to running
  ['] cold tcbs tcbaction + !		\ action of task 0 is COLD
  tcbs curtask !                        \ current task pointer
  080 task# c!                          \ current task 0, tasker disabled
  0 restart  ;				\ task 0 is Forth
\ multi-tasking version of key

\
\ In order for the Forth system to use a cooperative scheduler, PAUSE or
\ WAIT must be executed periodically. A natural place to insert PAUSE is
\ when polling the keyboard, or waiting for any other I/O to complete.
\
: mt-key        \ -- char ; schedules while waiting for character
  Begin  (key?) 0=                      \ while no character
    While  pause                        \ call scheduler
  Repeat                                \
  (key)  ;                              \ now get it

  assign mt-key to-do key               \ switch as action of KEY
\ semaphore

\
\ A SEMAPHORE is an extended variable used for signalling between tasks,
\ and for resource allocation.
\ The counter field is used as a count of the number of times the
\ resource may be used, and the arbiter field contains the TCB of the
\ task that last gained access. This field can be used for priority
\ arbitration and deadlock detection/arbitration.
\
: semaphore	\ -- ; -- addr [child]
  create  there ,			\ point to RAM
          0 ,(r)  0 ,(r)		\ count and arbiter fields
  does> @  ;

\
\ SIGNAL increments the counter field of a semaphore, indicating
\ either a) that another item has been allocated to the resource, or
\        b) it is available for use again, 0 indicating in use by a task
\
: signal	\ addr -- ; increment counter field of semaphore,
		\ so making it available for use by a task
  di  1 over +!  2+ off  ei  ;          \ inc. counter, release

\
\ REQUEST waits until the counter field of a semaphore is non-zero,
\ and then decrements the counter field by one.
\ This allows the semaphore to be used as a COUNTED semaphore. For
\ example a character buffer may be used where the semaphore counter
\ shows the number of available characters.
\ Alternatively the semaphore may be used purely to share resources.
\ The semaphore is initialised to one. The first task to REQUEST it
\ gains access, and all other tasks must wait until the accessing task
\ SIGNALs that it has finished with the resource.
\
: request	\ addr -- ; get access to resource, wait if count = 0
  begin
    dup  di  @ 0=                       \ n.b test and set
    while  ei pause                     \ operations must be
  repeat                                \ non-interruptible and indivisible
  -1 over +!                            \ got it, decrement counter
  curtask @ swap 2+ !			\ mark resource as mine
  ei  ;					\ re-enable interrupts
\ all done

decimal

;p
