\ ROM Forth serial i/o definitions         

((
Copyright (c) 1991

MicroProcessor Engineering
133 Hill Lane
Southampton SO1 5AF
England

tel: (+44) 703 631441
fax: (+44) 703 339691

First Release: 29/04/91

Changes:	See RELEASE.DOC for full details
))

\ Model definitions                                sfp 12/02/90

internal

\ Define serial i/o words in any suitable way

: ser-emit      \ char --
  (emit)  ;

: ser-key       \ -- char
  (key)  ;

: ser-key?      \ -- t/f
  (key?)  ;


\ File transfer constants                          sfp 26/01/90

Hex

\
\ characters important in these protocols
\
001 constant START
006 constant ACK
015 constant NAK

\
\ constants sent as triggers to host end of comms link
\
012 CONSTANT ^R                         \ read command code
017 CONSTANT ^W                         \ write
015 constant ^u                         \ $using
001 constant ^a                         \ use
006 constant ^f                         \ $from
005 constant ^e                         \ error

\ wait-ack                                         sfp 12/09/89

decimal

: wait-ack	\ -- ; wait for ACK character
  BEGIN
    ser-key                     \ read a char from serial line
    ACK =                       \ until it is an ACK
  UNTIL
;
\ This word waits for the host to send an ACK at the end of
\ part of a transfer.


\ wait-ack/nack                                    sfp 12/09/89

: wait-ack/nack         \ -- t/f ; true for NACK
  0
  begin
    drop                                \ drop old flag
    ser-key dup NAK = tuck              \ -- t/f char t/f
    swap ACK =  or                      \ -- t/f t/f'
  until  ;
\ This waits for either a NACK or an ACK from the host
\ and leaves true or false on stack.


\ send-block#                                      sfp 12/09/89

decimal

: send-block#           \ n -- ; send block number to server
  0 256 um/mod swap             \ split into high & low bytes
  ser-emit ser-emit             \ send block no., low byte first
  wait-ack                      \ wait for ack
;
\ Sends a single length number as two bytes 00-FF


\ synch-to-host                                    sfp 12/09/89

: synch-to-host         \ -- ; sync host to us
  BEGIN                         \ sync.
    ser-key                     \ get a char
    START =                     \ receive bytes until start code
  UNTIL                         \
  BEGIN
    ser-key?                    \ while there are keys still
  WHILE
    ser-key                     \ read them
    drop                        \ and drop them
  REPEAT                        \
  ACK ser-emit                  \ acknowledge as end of synch.
;
\ send$                                         R. sfp 26/01/90

: send$                 \ $addr ^char -- ;
  ser-emit                      \ send code
  synch-to-host                 \ synchronise
  dup 1+ swap                   \ addr+1 as base for reading
  c@ dup send-block#            \ send length of string
  0                             \ length - 0
  do dup i + c@                 \ get char at i+$addr
     ser-emit                   \ send char
  loop
  drop                          \ drop spare address
  ACK ser-emit                  \ send ack to complete transfer
  Wait-Ack/Nack Disk-error !    \ get status response
; 
\ This word sends a counted string and a trigger to the host
\ all done

external

decimal

;p
