\ Z84C15 ROM PowerForth startup code

((
released for use without royalty when used with the
MPE Z80/64180 Forth Cross Compiler.

Copyright (c) 1987-1991

MicroProcessor Engineering
133/133a Hill Lane
Shirley
Southampton SO1 5AF

tel: (+44) 703 63 14 41 (international)
     0703 631441        (domestic)

May 1991	Converted to text file

))
\ Version number

ascii 3 EQU MPE-REL		\ Major (structural) release
ascii 0 EQU MPE-VER		\ Minor release
ascii 0 EQU USRVER		\ Bug fix release

\ Start up table - entry points ...................sfp 28/03/90

hex
\
\ We assume that HERE points to an area that contains the CPU reset vector
\ and that space is available for a mode 2 interrupt table. By default this
\ table is 256 bytes long. The 256 byte boundary aligned base of this table
\ is a label called INT-PAGE. When interrupts are defined later the vectors
\ can be set by referring to an offset from INT-PAGE, e.g.
\
\	ENTRY-POINT INT-PAGE VECTOR# 2* + !
\
here 0FF00 and  equ int-page

\
\ in a ROMmed system it is convenient to keep the interrupt page at 0, and
\ to use the first 4 bytes for the reset vectors. Note that other locations
\ will be required for the RST nn entry points. This code should be added
\ by the user.
\
assembler
  di  ecld jp				\ disable interrupts, skip tables
forth

\
\ Reserve space for restart jumps & interrupt vectors. We assume that
\ the full 256 bytes for mode 2 is required
\
int-page 0100 + org

\
\ These two locations hold the words that will be executed at cold start
\ (RESET or COLD) and at warm start (WARM). Whatever word is inserted
\ between the ] and [ will be executed by the startup code below.
\
L: CLD1         ] startup [        \ cold start word
L: WRM1         ] abort [       \ warm start word

\
\ The Z80/64180 register set does not allow all of the Forth virtual
\ machine to be held in registers. The pointer to the USER area is held
\ in RAM in a location called UP.
\
there label UP
  2 allot-ram				\ reserve space for it

\
\ Cold and warm starts
\
\ This code starts the Forth virtual machine running.
\ Note that additional code is needed for the 84C15 to
\ set up the chip selects, wait states, etc. See the 84C15
\ manual for more details.
\
hex
0EE equ scrp		\ System Control Register Pointer
0EF equ scdp		\ System Control Data Pointer

\
\ This table contains initial values as address & data pairs
\ which are used by the initialisation code to set up the system
\ registers.
\
L: ITAB
  scrp c,  000 c,    scdp c,  01C c,	\ 0 i/o, 3 mem, 1 opcode wait states
  scrp c,  002 c,    scdp c,  097 c,	\ chip select 0 is 00000..07FFFh,
					\ chip select 1 is 08000..09FFFh
  scrp c,  003 c,    scdp c,  003 c,	\ set up misc reg.

\
\ Now actually set up the Z84C15 and start the Forth virtual machine
\
assembler
l: ecld
	hl, # itab ld   b, # 6 ld	\ table address, and no. of pairs
l: iloop
	c, (hl) ld   hl inc		\ get address
	a, (hl) ld   hl inc		\ get value
	(c), a out   iloop djnz		\ o/p & loop till done
        hl, # init-u0 ld   up hl ld     \ set user pointer
        iy, # init-r0 ld                \ set return stack
        sp, # init-s0 ld                \ set data stack
        bc, # cld1 ld                   \ set IP (see above)
l: snxt
        ix, # next ld   next,           \ pointer to next
					\ needed if space-saving NEXT, used
l: ewrm
        bc, # wrm1 ld   snxt jr
forth
\ all done

decimal

;p
