((
\ Umbilical Interpreter/Compiler                   J L 06/03/88

Jonathan Lee, February 1988

Copyright (c) 1988
MicroProcessor Engineering
133 Hill Lane
Shirley
Southampton SO1 5AF
tel: 0703 631441        (inside UK)
     (+44) 703 631441   (international)
))

decimal

only forth also c-c also umbilical definitions

\ ?Ram ..................................          sfp 14/05/91
Hex

: ?Ram          \ addr -- addr t/f
  Rom?                             \ ROM system ?
  If  the-label RAM-START @(s)l         \ yes, check RAM range
      over u> not                       \ RAM-START /> addr
      over the-equate EM @(s)l u> not   \ addr /> EM
      And                               \ Both conditions
  Else  -1                              \ in RAM always
  Endif  ;                              \

\ @(r>h)                                           sfp 28/06/90
\ These will become the actions of @(r) and c@(r) for RAM space
                                      \
: @(ur)         \ addr -- n
  symbol-of @ target-exec  ;          \ target executes own @

: C@(ur)        \ addr -- n
  symbol-of C@ target-exec  ;         \ target executes own C@

\ !(ur) c!(ur)                                     sfp 28/06/90
\ These will become the actions of @(r) and c@(r) for RAM space

: !(ur)         \ n addr --
  symbol-of ! target-exec  ;

: c!(ur)        \ addr -- n
  symbol-of c! target-exec  ;

\ c,(ur) ,(ur)                                     sfp 04/04/89
\ These will become the actions of c,(r) and ,(r)

: c,(ur)                              \ n -- ; in target RAM
  there(t) c!(ur)                     \ address to be used
  1 allot-ram  ;                      \ reserve space
                                      \
: ,(ur)                               \ n -- ; in target RAM
  there(t) !(ur)                      \ address to be used
  2 allot-ram  ;                      \ reserve space

\ @(u) !(u) .............................          sfp 04/04/89
\ These will become the actions of @(t) and !(t) for CODE space
                                      \
: @(u)                                \ addr -- n
  ?ram                                \ If RAM
  If  @(ur)                           \ do it down link
  Else  emu-@(u)                      \ or use LeProm emulator
  Endif  ;                            \
                                      \
: !(u)                                \ n addr --
  ?ram                                \
  If  !(ur)                           \ target executes own !
  Else  emu-!(u)  Endif  ;            \ use LeProm emulator

\ c!(u) c@(u) ...........................          sfp 04/04/89
\ These will be the actions of c@(t) and c!(t) for CODE space

: c!(u)                               \ c addr --
  ?ram                                \
  If  c!(ur)                          \ target executes own c!
  Else  emu-c!(u)  Endif  ;           \ use LeProm emulator
                                      \
: c@(u)                               \ addr -- c ( On TARGET )
  ?ram                                \
  If  c@(ur)                          \ do it down link
  Else  emu-c@(u)                     \ use LeProm emulator
  Endif  ;                            \

\ error(u) ?value(s)                               sfp 13/01/90

: error(u)      \ -- ; error handling
  disp-error                            \ display it
  only forth also c-c                   \ force host vocabs
  also umbilical definitions            \
  abort  ;                              \

: ?value(s)     \ symbol-addr -- value
  2dup resolved? ?undef                  \ check symbol exists
  @(s)l  ;                               \ get its value

: umbilical-.log        \ $addr addr --
  2drop  ;

: (umbilical-.log)        \ $addr addr sym(d) --
  2drop 2drop  ;

\ ?sci ?scc .............................          sfp 06/04/89
                                      \
: ?sci                                \ Special INTERPRETER
  sci context-find                    \
  IF  execute  r> drop  ENDIF         \
;                                     \
                                      \
: ?scc                                \ Special COMPILER
  scc context-find                    \
  IF  execute  r> drop  ENDIF         \
;                                     \

\ ?target(u)                                       sfp 06/04/89

: ?target(u)    \ --
  cs> 'word find(t)                    \ Word exists ?
  IF  compil?                           \ yes, compiling?
      IF  @(s)l ,(t)                     \ yes, lay CFA
      ELSE  2dup voc?                    \ is it a vocabulary
            If    is-voc(s)             \ yes, set vocab
            Else  target-exec           \ no, execute target CFA
            Endif                       \
      ENDIF                             \
      r> drop                           \ exit outer word
  ENDIF  ;                              \

\ ?equates(u)                                      sfp 28/06/90

: ?EQUATES(u)   \ -- ; identical to ?EQUATES
  cs> 'word 'Equates(s) 2@(g) voc-chunk+ @(s)l latest-chunk @(p)l v-find(s)
  IF  @(s)l  COMPIL?                    \ yes, compiling ?
      IF  LITERAL  ENDIF                \ yes, lay literal
      R> DROP                           \ Leave outer word
  ELSE  DROP                            \ discard text addr
  ENDIF  ;                              \

\ ?labels                                          sfp 28/06/90

: ?LABELS(u)
  cs> 'word 'Labels(s) 2@(g) voc-chunk+ @(s)l latest-chunk @(p)l v-find(s)
  IF  ?value(s)  COMPIL?                \ compiling ?
      IF  symbol-of LIT ?value(s) ,(t)  \ yes, compile LIT
          ,(t)                          \   and lay value
      ENDIF
      R> DROP                           \ leave outer word
  ELSE  DROP                            \ discard text addr
  ENDIF  ;                              \

\ number                                           sfp 06/04/89

: Number        \ -- n | abort
  here number? ?undef  ;

\ Outer Interpreter ........................... J Lsfp 28/06/90

: INTERPRETING(u)
  ?SCI ?INTERPRETER ?EQUATES ?LABELS(u)
  ?TARGET(u) NUMBER  ;

: COMPILING(u)
  ?SCC ?COMPILER ?TARGET(u) ?EQUATES ?LABELS(u)
  NUMBER COMPILE-NUM  ;

: ASSEMBLING(u)
  ?ASSEMBLER ?SCI ?INTERPRETER ?EQUATES ?LABELS(u)
  ?TARGET(u) ?C-C NUMBER ;

\ interpret(u)  ............................... J Lsfp 29/06/90

: interpret(u)          \ --
  BEGIN
    get-token c@
    WHILE  compil?
           IF   compiling(u)
           ELSE interp?
                if    interpreting(u)
                else  assembling(u)
                endif
           ENDIF
           ?STACK
  REPEAT  ;

\ Quit ........................................ J Lsfp 29/06/90
                                      \
: Quit-emu                            \ --
  s0 @ sp!                            \ bullet proofing
  STATE(T) off(g)                     \ force interpretation
  blk off                             \ input from keyboard
  BEGIN                               \
    r0 @ rp!                          \ N.B. host return stack
    cr query interpret(u)             \ deal with line
    interp?                           \ see if prompt needed
    IF  prompt  ENDIF                 \
  AGAIN                               \
;                                     \

\ Reset-xc .................................... J Lsfp 19/09/89

: Reset-xc                            \ --
  no-log  internal                    \ no log, no heads
  assign emu-c@(u) to-do c@(o)  assign emu-c!(u) to-do c!(o)
  assign emu-@(u) to-do w@(o)   assign emu-!(u) to-do w!(o)
\ use emu-xx(u) for Harvard (e.g. 8031), xx(u) for others
  assign c@(u) to-do c@(t)  assign c!(u) to-do c!(t)
  assign @(u)  to-do @(t)   assign !(u)  to-do !(t)
  assign @(u)  to-do w@(t)  assign !(u)  to-do w!(t)
  Harvard?
  If
    assign c@(ur) to-do c@(x) assign c!(ur) to-do c!(x)
    assign @(ur)  to-do @(x)  assign !(ur)  to-do !(x)
    assign c,(ur) to-do c,(r) assign ,(ur)  to-do ,(r)
  Endif
  dp-in-ram @				\ dp in ram selected
  If					\
    there(t) org			\
  Endif					\
  assign quit-emu to-do quit		\ error recovery XCompiler
  assign error(u) to-do error  assign (abort) to-do abort  ;

\ targ-prompt                                      sfp 28/06/90

: targ-prompt   \ -- ; prompt while Umbiiical Forth runs
  ."  ok"  depth ?dup
  if  ." -" .  endif
  space  ;

\ cord ........................................ J Lsfp 29/06/90
decimal                                 \
: cord                                  \ --
  assign umbilical-.log to-do .log	\ switch log
  assign (umbilical-.log) to-do (.log)	\ switch (log)
  assign interpret(u) to-do interpret(t)  \ and interpreter
  assign targ-prompt to-do prompt	\
  reset-xc                              \ reset re-usable code
  cls  cr cr                            \ pretties
  cr ." Umbilical Forth V2.00"          \ sign on
  cr .target                            \ say which target
  cr ." Copyright (C) 1990 MicroProcessor Eng. Ltd."
  cr  decimal                           \ force base
  cr ." BASE is now DECIMAL"            \ warn user
  cr cr cr ."   ok"                     \ makes it prettier
  quit-emu  ;                           \ run interpreter

\ Umbilical-Forth ............................. J Lsfp 06/07/90

also c-c definitions                  \

: Umbilical-Forth	\ -- ; directive
  (finis)				\ finish cross compilation
  umbilical-mode  In-Umbilical On(g)	\ install XShell2 API
  assign (abort) to-do abort		\ reset error handler
  only forth also c-c also umbilical	\ set search order
  cls cr cr ." Reset target and then "	\
  ." press <space> to run Umbilical Forth " \ wait...
  Begin  key bl =  Until		\ poll for space bar
  Ser-close Ser-open			\ initialise serial line
  close-image				\
  >emulator				\
  cord					\ run umbilical forth
;

previous definitions                  \

\ more                                             sfp 17/05/91

: more          \ --
  begin  key?  while  key drop  repeat  \ flush input buffer
  cr ." Press <space> to continue "     \ ask for input
  key bl <> abort" ABORTED!"  ;         \ wait for it

: words(u)      \ addr(s) -- ; display sumbol names
  context(s) 2@(p) 2swap is-voc(s)       \ set new context
  words(s)  cr                          \ show names
  is-voc(s)  ;                          \ restore context

\ Special INTERPRETER ......................... J Lsfp 13/01/90
                                      \
only Forth also c-c also Umbilical also SCI definitions
                                      \
: Back          cc/si                 \ -- ; host Forth system
  assign .depth to-do prompt          \ reset host
  assign (quit) to-do quit            \
  assign (abort) to-do abort          \
  assign (error) to-do error          \
  only forth  also c-c                \ force search order
  also umbilical definitions          \
  abort  ;                            \ normal Forth system

\ Words                                            sfp 06/07/90

: Words         cc/si   \ --
  words(s) cr  ;

: Help          cc/si   \ --
  Cr ." Special when Interpreting  "    \ Special Case
  Cr ." ========================="      \ Interpreter
  SCI words  more                       \
  INTERPRETER words  more
  Cr ." Special when Compiling"         \ then Special Case
  Cr ." ======================"         \ Compiler
  COMPILER words
  Cr  ;

\ labels equates                                   sfp 17/05/91

: labels        cc/si   \ -- ; show label names
  'labels(s) 2@(g) words(u)  ;

: equates       cc/si   \ -- ; show equate names
  'equates(s) 2@(g) words(u)  ;

\ Special case INTERPRETER ..................... J sfp 28/06/90

: .             cc/si   \ n --
  .  ;

: u.            cc/si   \ n --
  u.  ;

: ?             cc/si   \ addr --
  symbol-of @ target-exec  .  ;

\ Special case INTERPRETER ..................... J sfp 04/04/89
                                      \
decimal                               \
                                      \
: Hex           cc/si                 \ --
  16 base !  ;                        \
                                      \
: Decimal       cc/si                 \ --
  10 base !  ;                        \
                                      \
: Binary        cc/si                 \ --
  2 base !  ;                         \

\ Special case INTERPRETER cont. ................  sfp 13/01/90

: Bye           cc/si   \ -- ; leave system
  Bye  ;                      \ be kind to DOS

: Cls           cc/si   \ -- ; clear the screen
  cls  ;

\ dump                                             sfp 14/05/91
hex

: dump          cc/si   \ addr len -- ; dump DATA space
  BASE @ -rot  HEX  bounds              \ get limits
  DO  CR  i .WORD  i 010 bounds         \ print address
      do  space  i c@(t) .BYTE  loop    \ print byte
      2 SPACES  I 010 bounds            \ gap
      DO  I c@(t) .ASCII  LOOP          \ ascii o/p
      halt? ?leave                      \ quit or halt
  010 +LOOP                             \ until done
  BASE !  CR  ;                         \ restore base

\ latest                                           sfp 28/06/90

: LATEST        cc/si  \ --
  1 abort" No NAMEs on target"  ;

\ fill erase blank                                 sfp 28/06/90

: fill          cc/si   \ addr len char --
  fill(t)  ;				\ reorder and fill

: erase         cc/si   \ addr len --
  0 fill(t)  ;

: blank         cc/si   \ addr len --
  bl fill(t)  ;

\ Special case INTERPRETER cont. ................  sfp 29/06/90

: Load          cc/si   \ n --
  BLK @ >R  >IN @ >R  >IN off  BLK !    \ save context
  interpret(u)                          \ interpret it
  R> >IN !  R> BLK !  ;                 \
                                        \
: Thru          cc/si   \ n1 n2 --
  BLK @ >R  >IN @ >R                    \ save old
  1+ swap                               \ massage for DO
  ?do  >IN off  i BLK !                 \ reset context
       interpret(u)                     \ interpret it
  loop                                  \
  R> >IN !  R> BLK !  ;                 \ restore context

\ Special case INTERPRETER cont. ................  sfp 29/06/90


: Load-Using  cc/si     \ screen# -- ; <file-name>
  cr cs> get-token $using
  If  re-open 8 error  Endif
  cr .curr-file cr
  [ also sci ] LOAD [previous]
  cr ." Finished with " .scr cr re-using  ;

: Thru-Using  cc/si     \ first# last# -- ; <file-name>
  cr cs> get-token $using
  If  re-open 8 error  Endif
  cr .curr-file cr
  [ also sci ] THRU [previous]
  cr ." Finished with " .scr cr re-using  ;

\ Special case INTERPRETER cont. ................  sfp 29/06/90


: From-file     cc/si   \ first last -- ; <file-name>
  ['] interpret	>body @	>r		\ switch interpreter
  assign interpret(u) to-do interpret	\
  cs> get-token $from			\ go process it
  r> ['] interpret >body !		\ old interpreter
;

: From          cc/si   \ first last --
  ['] interpret	>body @	>r		\ switch interpreter
  assign interpret(u) to-do interpret	\
  curr-text$ $from			\ go process it
  r> ['] interpret >body !		\ old interpreter
;

\ Special case INTERPRETER cont. ................  sfp 28/06/90

: All           cc/si   \ -- 1 -1 ; to target stack
  1 -1  ;

: Onwards       cc/si   \ n -- n -1 ; on target stack
  -1 ;

: Alone         cc/si   \ n -- n n
  dup  ;

\ Special case INTERPRETER cont. ................  sfp 28/06/90

: UpTo          cc/si   \ n -- 1 n
  1 swap  ;
((
: List          cc/si   \ n --
  List  ;
))
\ .fth Use Qc  Using L N P QX >SCR                 sfp 12/01/90

: .FTH          cc/si  .fth  ;
: Use           cc/si  Use  ;
: QC            cc/si  QC  ;

: Using         cc/si  Using  ;
((
: L             cc/si  scr @ List  ;
: N             cc/si  scr @ 1+ List  ;
: P             cc/si  scr @ 1- List  ;
: QX            cc/si  Qx  ;
: .SCR          cc/si  .SCR  ;
))
\ .s \ ( ((                                        sfp 05/06/91

: .S            cc/si
  .s  ;

: \             cc/si  [compile] \  ;
: (             cc/si  [compile] (  ;
: ((            cc/si  [compile] ((  ;

\ End of special case INTERPRETER                  sfp 03/04/89

only forth definitions

